from math import*
import numpy as np



def Air_xH2O(T_db=None,T_wb=None,RH=None,w=None,h=None):
      Mdry=28.965
      Mv=18.01528
      if (T_wb is not None and w is None):
        w=Air_w(T_db=T_db,T_wb=T_wb)
      if RH is not None:
        w=Air_w(T_db=T_db,RH=RH)     
      xH2O=w*Mdry/(w*Mdry+Mv*1000)
      return round(xH2O,4)

def Air_Pv_sat(T_db) :

#calcul de la pression de vapeur sat
# référence : 2013 ASRHAE Handbook—Fundamentals (SI) - CHAPTER 1 - PSYCRHOMETRICS - Equations (5) and (6) - Hyland and Wexler 1983 equations
# Pv_sat = saturation pressure, Pa
# Tk = absolute temperature, K = °C + 273.15


	C1 = -5.6745359 * 10 ** 3
	C2 = 6.3925247 * 10 ** 0
	C3 = -9.677843 * 10 ** (-3)
	C4 = 6.2215701 * 10 ** (-7)
	C5 = 2.0747825 * 10 ** (-9)
	C6 = -9.484024 * 10 ** (-13)
	C7 = 4.1635019 * 10 ** (0)
	C8 = -5.8002206 * 10 ** (3)
	C9 = 1.3914993 * 10 ** (0)
	C10 = -4.8640239 * 10 ** (-2)
	C11 = 4.1764768 * 10 ** (-5)
	C12 = -1.4452093 * 10 ** (-8)
	C13 = 6.5459673 * 10 ** (0)

	Tk = T_db + 273.15

	if T_db < 0 : #valable entre -100 et 0 °C

		Pv_sat = exp(C1 / Tk + C2 + C3 * Tk + C4 * Tk ** 2 + C5 * Tk ** 3 + C6 * Tk ** 4 + C7 * log(Tk))
	else :  #valable entre 0 et 200 °C

		Pv_sat = exp(C8 / Tk + C9 + C10 * Tk + C11 * Tk ** 2 + C12 * Tk ** 3 + C13 * log(Tk))
		




	return Pv_sat




def Air_RH(w=None, P=101325.0, Pv_sat=None, T_db=None, T_wb=None, h=None):
    """
    Calcule l'humidité relative (RH) en fonction des paramètres donnés.

    Args:
        w (float): Humidité absolue (g/kg).
        P (float): Pression atmosphérique en Pa (par défaut 101325 Pa).
        Pv_sat (float): Pression de vapeur saturée en Pa (facultatif si T_db est fourni).
        T_db (float): Température de bulbe sec en °C.
        T_wb (float): Température de bulbe humide en °C (facultatif).
        h (float): Enthalpie de l'air humide (facultatif).

    Returns:
        float: Humidité relative (RH) en pourcentage (%), arrondie à 2 décimales.
    """
    if w is not None and h is not None and T_db is None:
        # Si `w` n'est pas fourni mais `h` et `T_db` sont disponibles, calculer T_db.
        T_db = Air_T_db(w=w, h=h)
       
    
    if Pv_sat is None:
        if T_db is None:
            pass
        else:
            # Calculer Pv_sat à partir de T_db.
            Pv_sat = Air_Pv_sat(T_db)
    
    if w is None:
        if T_wb is None:
            raise ValueError("Either w or T_wb must be provided.")
        else:
            # Calculer `w` à partir de T_wb en supposant RH=100%.
            w_w = Air_w(RH=100, T_db=T_wb)  # Humidité absolue à saturation.
            h = Air_h(T_db=T_wb, w=w_w)    # Enthalpie à T_wb et w_w.
            w = Air_w(h=h, T_db=T_db)      # Humidité absolue pour T_db avec enthalpie h.
    
    # Calcul de la pression partielle de la vapeur d'eau (Pv).
    Pv = P * (w / 1000) / ((w / 1000) + 0.62198)
    
    # Calcul de l'humidité relative (RH).
    RH = (Pv / Pv_sat) * 100

    if Pv_sat is not None and w is not None:
        Pv = P * (w / 1000) / ((w / 1000) + 0.62198)
        RH = (Pv / Pv_sat) * 100

    if T_db is not None and w is not None:
        Pv = P * (w / 1000) / ((w / 1000) + 0.62198)
        RH = (Pv / Air_Pv_sat(T_db)) * 100
    
    return round(RH, 2)



def Air_w(RH=None, P=101325.0, Pv_sat=None, T_db=None, h=None, T_wb=None):
    # Calcul de la pression de vapeur saturée (Pv_sat) si elle n'est pas fournie

 
    if Pv_sat is None and T_db is not None:
       Pv_sat = Air_Pv_sat(T_db)

    # Calcul de l'humidité absolue à partir de l'humidité relative
    if RH is not None and Pv_sat is not None:
        Pv = Pv_sat * (RH / 100)
        
        if Pv < P:
            w = 0.62198 * Pv / (P - Pv) * 1000
        else:
            w = None

    # Calcul de l'humidité absolue à partir de la température de bulbe humide (T_wb)
    if T_wb is not None:
        w_w = Air_w(RH=100, T_db=T_wb)  # Calculer l'humidité absolue à saturation (RH = 100)
        h = Air_h(T_db=T_wb, w=w_w)     # Calculer l'enthalpie de l'air humide à T_wb et w_w
        w = Air_w(h=h, T_db=T_db)       # Calculer w pour la température de bulbe sec T_db avec l'enthalpie h

    # Calcul de l'humidité absolue à partir de l'enthalpie (h) si elle est fournie
    if h is not None and T_db is not None:
        w = ((h - 1.006 * T_db) / (2501 + 1.0805 * T_db)) * 1000

    # Cas où T_db est inconnu, mais h et RH sont fournis
    if T_db is None and h is not None and RH is not None:
        tolerance = 0.1

        # Fonction à résoudre : écart entre l'enthalpie calculée et l'enthalpie fournie
        def equation(T_db):
            # Calcul de la pression de vapeur saturée Pv_sat à T_db
            Pv_sat = Air_Pv_sat(T_db)

            # Calcul de l'humidité absolue w à partir de RH et T_db
            Pv = Pv_sat * (RH / 100)
            w = 0.62198 * Pv / (P - Pv) * 1000

            # Calcul de l'enthalpie à partir de w et T_db
            h_calc = 1.006 * T_db + (w * (2501 + 1.0805 * T_db)) / 1000

            # Retourne l'écart entre l'enthalpie calculée et l'enthalpie fournie
            return h_calc - h
        
        # Estimation initiale pour T_db (on peut commencer avec 20°C)
        T_db_initial_guess = 20
        
        # Utilisation de fsolve pour résoudre l'équation
        T_db_solution = fsolve(equation, T_db_initial_guess, xtol=tolerance)

        # fsolve retourne un tableau, donc il faut accéder à la première valeur
        T_db_solution = T_db_solution[0]

        # Calcul de l'humidité absolue correspondante
        Pv_sat = Air_Pv_sat(T_db_solution)
        Pv = Pv_sat * (RH / 100)
        w = 0.62198 * Pv / (P - Pv) * 1000
        ################""
    if w is not None:
        w=round(w,3)
    return w


# def Air_h(T_db=None, w=None, T_wb=None):
# 	if (T_db is not None and w is not None) :
# 		Enthalpie = 1.006 * T_db + (w / 1000) * (2501 + 1.0805 * T_db)
#     elif (T_db is not None and T_wb is not None) :
# 		w=Air_w(T_db=T_db, T_wb=T_wb)
# 		Enthalpie = 1.006 * T_db + (w / 1000) * (2501 + 1.0805 * T_db)
# 	else:
# 		Enthalpie=None
# 	return Enthalpie
def Air_T_wb(T_db=None, RH=None, P=101325.0,w=None,h=None):
    """
    Calcule la température de bulbe humide (T_wb) en utilisant les équations d'enthalpie et d'humidité.
    """
    if (w is not None and h is not None):
        T_db=Air_T_db(w=w,h=h)

    # if T_db is None or RH is None:
    #     raise ValueError("T_db et RH sont requis pour calculer T_wb.")

    # Humidité absolue à T_db et RH
    if w is None:
        w = Air_w(RH=RH, P=P, T_db=T_db)

    # Fonction à résoudre pour trouver T_wb
    def equation(T_wb):
       
        w_wb = Air_w(RH=100, P=P, T_db=T_wb)  # Humidité absolue à saturation à T_wb
        h = 1.006 * T_db + (w / 1000) * (2501 + 1.0805 * T_db)  # Enthalpie à T_db
        h_wb = 1.006 * T_wb + (w_wb / 1000) * (2501 + 1.0805 * T_wb)  # Enthalpie à T_wb
        return h - h_wb

    # Résolution numérique pour T_wb
    T_wb_initial_guess = T_db - 5  # Devine initiale pour la résolution
    T_wb_solution = fsolve(equation, T_wb_initial_guess)[0]

    return round(T_wb_solution,2)



def Air_h(T_db=None, w=None, T_wb=None, RH=None):
    # Si T_db et w sont fournis
    if RH is not None:
        w=Air_w(T_db=T_db,RH=RH)
    
    # Si T_db et T_wb sont fournis
    if T_wb is not None:
        w = Air_w(T_db=T_db, T_wb=T_wb)  
       
    if T_db is not None and w is not None:
        Enthalpie = 1.006 * T_db + (w / 1000) * (2501 + 1.0805 * T_db)
    
    # Si aucune des conditions précédentes n'est remplie
    else:
        Enthalpie = None
    
    return round(Enthalpie,3)


     


# 	return rho_hum
def Air_rho_dry(T_db=None, RH=None, P=101325, T_wb=None,w=None,h=None):
    if (T_db is not None and w is not None) or (h is not None and w is not None):
        rho_dry=Air_rho_hum(T_db=T_db,w=w,h=h)/(1+w/1000)
    if (T_db is not None and RH is not None) or (T_db is not None and T_wb is not None):
        rho_dry=Air_rho_hum(T_db=T_db,RH=RH,T_wb=T_wb,)/(1+Air_w(RH=RH,T_db=T_db,T_wb=T_wb,)/1000)
    return rho_dry

def Air_v_dry(T_db=None, RH=None, P=101325, T_wb=None,w=None,h=None):
    v_dry=1/Air_rho_dry(T_db=T_db, RH=RH, P=P, T_wb=T_wb,w=w,h=h)
    return v_dry

def Air_rho_hum(T_db=None, RH=None, P=101325, T_wb=None,w=None,h=None):
    if (w is not None and h is not None):
        T_db=Air_T_db(w=w,h=h)
        RH=Air_RH(w=w,h=h)
    if RH is None and T_wb is not None:
        RH = Air_RH(T_db=T_db, T_wb=T_wb)  # Calculer RH si T_wb est fourni
    
    if (T_db is not None and RH is None and w is not None):
        RH=Air_RH(T_db=T_db, w=w)
    
    Tk = T_db + 273.15  # Conversion en Kelvin
    Rv = 461  # Constante des gaz pour la vapeur d'eau (J/(kg·K))
    Ra = 287.66  # Constante des gaz pour l'air sec (J/(kg·K))
    Psat = Air_Pv_sat(T_db)  # Pression de vapeur saturée à T_db
    
    # Pression partielle de vapeur d'eau
    Pv = Psat * (RH / 100)
    
    # Masse volumique des constituants
    rho_v = Pv / (Rv * Tk)  # Masse volumique de la vapeur d'eau
    rho_a = (P - Pv) / (Ra * Tk)  # Masse volumique de l'air sec
    
    # Calcul de la constante des gaz pour l'air humide
    Rah = Ra / (1 - ((RH / 100) * Psat / P) * (1 - Ra / Rv))
    
    # Masse volumique de l'air humide
    #rho_hum=(P/(Rah*Tk))
    #print('rho_hum 1',rho_hum)
    #rho_hum=(1-((1-Ra/Rv)*Pv)/P)*P/(Ra*Tk)
    #print('rho_hum 2',rho_hum)
    rho_hum = (rho_a * Ra + rho_v * Rv) / Rah
    #print('rho_hum 3',rho_hum)
    
    return rho_hum



def Air_v_hum(T_db=None, RH=None, P=101325,T_wb=None,w=None,h=None):
    rho_hum=Air_rho_hum(T_db=T_db, RH=RH, P=P,T_wb=T_wb,w=w,h=h)
    v_hum=1/rho_hum
    return v_hum





from scipy.optimize import fsolve
from math import exp, log

def Air_T_db(RH=None, w=None, h=None, Pv_sat=None, P=101325):
    if RH is not None and w is not None:
        # Calcul de T_db à partir de RH et w
        def equations(T_db):
            Pv_sat_calculated = Air_Pv_sat(T_db)
            Pv = P * (w / 1000) / (0.62198 + w / 1000)
            return Pv_sat_calculated * RH / 100 - Pv

        T_db_initial_guess = 20.0
        T_db_solution = fsolve(equations, T_db_initial_guess)
        return round(T_db_solution[0],3)

    elif h is not None and w is not None:
        # Calcul de T_db à partir de h et w
        T_db_solution=(h - (w / 1000) * 2501) / (1.006 + (w / 1000) * 1.0805)
        return round(T_db_solution,2)

    elif Pv_sat is not None:
        # Calcul de T_db à partir de Pv_sat
        def inverse_Pv_sat(T_db):
            return Air_Pv_sat(T_db) - Pv_sat

        T_db_initial_guess = 20.0
        T_db_solution = fsolve(inverse_Pv_sat, T_db_initial_guess)[0]
        return round(T_db_solution,3)

    else:
        raise ValueError("Insufficient parameters provided for T_db calculation")


##################""

def Air_T_wb_ROLAND_STULL(Td, RH) :   #calcul de la temp du bulbe humide
	Tw = Td * atan(0.151977 * (RH + 8.313659) ** (1 / 2)) + atan(Td + RH) - atan(RH - 1.676331) + 0.00391838 * (RH) ** (3 / 2) * atan(0.023101 * RH) - 4.686035
#Tw = 20 * atan(0.151977 * (50 + 8.313659) ** (1 / 2)) + atan(20 + 50) - atan(50 - 1.676331) + 0.00391838 * (50) ** (3 / 2) * atan(0.023101 * 50) - 4.686035
	return round(Tw,3)

#Wet-Bulb Temperature from Relative Humidity and Air Temperature
#ROLAND STULL
#University of British Columbia, Vancouver, British Columbia, Canada
#(Manuscript received 14 July 2011, in final form 28 August 2011)


# def w(Pv_sat, RH, P):



# 	Pv = Pv_sat * (RH / 100)
# 	w= 0.62198 * Pv / (P - Pv) * 1000


# 	return w








def T_sat(w_target):

	T = -100
	Erreur = Air_w(Air_Pv_sat(T), 100) - w_target


	while Erreur <= 0 :
		T = T + 0.02
		Erreur = Air_w(Air_Pv_sat(T), 100) - w_target



	T_sat = T

	return T_sat





from scipy.optimize import fsolve

def Air_T_dp(w=None, P=101325,T_wb=None, T_db=None, RH=None, h=None):
    """
    Calcule la température de rosée en fonction de w_target (rapport d'humidité) et P (pression atmosphérique).
    """


    if (w is None and T_wb is not None and T_db is not None):
        w=Air_w(T_wb=T_wb, T_db=T_db)
    
    if (w is None and RH is not None and T_db is not None):
        w=Air_w(RH=RH, T_db=T_db)

    if (w is None and h is not None and T_db is not None):
        w=Air_w(h=h, T_db=T_db)
    
    if (w is None and h is not None):
        w=Air_w(h=h, RH=100)
    

    # Fonction d'erreur pour trouver T où Pv_sat(T) correspond à w
    def error_function(T):
        Pv_sat = Air_Pv_sat(T)
        w_calc = 0.62198 * Pv_sat / (P - Pv_sat) * 1000
        return w_calc - w

    # Résolution avec une estimation initiale
    T_initial_guess = 10.0  # Vous pouvez ajuster cela selon les besoins
    T_rosee_solution = fsolve(error_function, T_initial_guess)

    return round(T_rosee_solution[0],2)











# -*- coding: utf-8 -*-
"""
Created on Sat Sep  5 19:39:26 2020

@author: zohei
"""

from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.utils import dumpException

#############les modèles d'un groupe frigorifique#################
#from AHU import FreshAir
#from AHU import HeatingCoil
#from AHU.Humidification import Humidifier
from AHU.Humidification import Humidifier


from AHU.Connect import Air_connect
from AHU.AirPort.AirPort import AirPort




class CalcHMDContent(QDMNodeContentWidget):
    def initUI(self):
        
        self.HumidType_lbl = QLabel("Type d'humidification", self)
        self.HumidType=QComboBox(self)
        self.HumidType.addItem("adiabatique")
        self.HumidType.addItem("vapeur")
        self.HumidType.view().setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded) 
        
        
        self.P_drop_lbl=QLabel("Perte de pression (bar)", self) 
        self.P_drop_edit=QLineEdit("0.001", self)
        
        self.HA_target_lbl=QLabel("w cible (g/kgas)", self) 
        self.HA_target_edit=QLineEdit("12", self)
        
        
        
        # self.Tref_lbl=QLabel("refoidissement cible (°C)", self) 
        # self.Tref_edit=QLineEdit("80", self)
        
        self.m_water_lbl_title = QLabel("débit d'eau g/s", self)
        self.m_water_lbl = QLabel("", self)
        
        # self.Qlosses_lbl_title = QLabel("Energie dissipée (kW):", self)
        # self.Qlosses_lbl = QLabel("", self)
        
        # self.Tis_lbl_title = QLabel("Temp. isentrop. (°C)", self)
        # self.Tis_lbl = QLabel("", self)
        
        
        
        
         
        self.layout=QVBoxLayout()

        self.layout.addWidget(self.HumidType_lbl)
        self.layout.addWidget(self.HumidType)
        
        self.layout.addWidget(self.P_drop_lbl)
        self.layout.addWidget(self.P_drop_edit) 
        
        # self.layout.addWidget(self.Tref_lbl)
        # self.layout.addWidget(self.Tref_edit)
        
        self.layout.addWidget(self.HA_target_lbl)
        self.layout.addWidget(self.HA_target_edit)
        
        
        
        self.layout.addWidget(self.m_water_lbl_title)
        self.layout.addWidget(self.m_water_lbl)  
        
        # self.layout.addWidget(self.Qlosses_lbl_title)
        # self.layout.addWidget(self.Qlosses_lbl) 
      
        # self.layout.addWidget(self.Tis_lbl_title)
        # self.layout.addWidget(self.Tis_lbl)
        
     
        
                         
        
        self.setLayout(self.layout)
        
        self.layout.setAlignment(Qt.AlignRight)
        self.layout.setObjectName(self.node.content_label_objname)
        
    def serialize(self):
        res = super().serialize()
        res['value'] = self.P_drop_edit.text()
        res2 = super().serialize()
        res2['value2'] = self.HA_target_edit.text()
        
        res3 = super().serialize()
        res3['value3'] = self.HumidType.currentText()
        
        
        
        # res4 = super().serialize()
        # res4['value4'] = self.F_kgs_edit.text()
        
        return res,res2,res3 #,res4

    def deserialize(self, data, hashmap={}):
        res = super().deserialize(data, hashmap)
        res2 = super().deserialize(data, hashmap)
        res3 = super().deserialize(data, hashmap)
        # res4 = super().deserialize(data, hashmap)
        # print("res=",res,res2,res3,res4)
        # print("dataaaaaaaaaa=",data)
        try:
            
            value = data[0]["value"]
            value2 = data[1]['value2']
            value3 = data[2]['value3']
           # value4 = data[3]['value4']
            
            # print("values=",value,value2,value3,value4)
            
            self.P_drop_edit.setText(value)
            self.HA_target_edit.setText(value2)
            self.HumidType.setCurrentText(value3)
            # self.F_kgs_edit.setText(value4)
            
            return True & res  & res2 & res3 #& res4
        except Exception as e:
            dumpException(e)
        return res ,res2,res3 #,res4

@register_node(OP_NODE_HMD)
class CalcNode_HMD(CalcNode):
    icon = "icons/HMD.png"
    op_code = OP_NODE_HMD
    op_title = "Humidifier"
    content_label = "/"
    content_label_objname = "calc_node_HMD"
    
    def __init__(self, scene):
        super().__init__(scene, inputs=[2], outputs=[1])
        self.eval()
        
    def initInnerClasses(self):
        self.content = CalcHMDContent(self)
        self.grNode = CalcGraphicsNode(self)
        
        
        self.grNode.height=250
        self.grNode.width=200
        
        self.content.P_drop_edit.textChanged.connect(self.onInputChanged)
#        self.content.Tref_edit.textChanged.connect(self.onInputChanged)
        self.content.HA_target_edit.textChanged.connect(self.onInputChanged)
        self.content.HumidType.currentIndexChanged.connect(self.onInputChanged)
  
    def evalOperation(self, input1, input2):
        self.value=[]
        
        a=AirPort()
        a.w=input1[0]
        a.F=input1[1]
        a.P=input1[2]*1e5
        a.h=input1[3]
       
        HMD=Humidifier.Object()
        
        
        Air_connect(HMD.Inlet,a)
        HumidType=self.content.HumidType.currentText()
       # print("HumidType=",HumidType)
        
        HMD.wo_target=float(self.content.HA_target_edit.text())
        print("HMD.wo_target",HMD.wo_target)
        HMD.HumidType=HumidType
        HMD.calculate()
        
        
       
        ####################
        HMD.P_drop=1e5*float(self.content.P_drop_edit.text())
       
        #HMD.Tdischarge_target=float(self.content.Tref_edit.text())
        HMD.calculate()
        
       
        self.value.append(HMD.Outlet.w) #HAe
        self.value.append(HMD.Outlet.F) #débit
        self.value.append(HMD.Outlet.P/1e5) #pression min
        self.value.append(HMD.Outlet.h) #Enthalpie
        
        self.content.m_water_lbl.setText("%.2f" % (HMD.F_water*1000)) #"%d" % 
        # #self.content.Qlosses_lbl.setText("%f" % (COMP.Q_losses/1000))
        # #self.content.Tis_lbl.setText("%f" % (HMD.Tis-273.15))
        # #self.content.lbl.setText("%f" % val[3])
        
        
            
        return self.value


from datetime import date
import json
import os
from dateutil.parser import parse
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle

current_directory = os.path.dirname(os.path.abspath(__file__))
coefficients_file_path = os.path.join(current_directory, 'coefficients_sonalgaz_elec.json')

class input_Facture:
    def __init__(self, start, end, 
                 kWh_pointe=0, kWh_pleine=0, kWh_jour=0, 
                 kWh_nuit=0, kWh_hors_pointe=0, kWh_poste_unique=0,
                 kvarh_reactif=0,
                 PMA_kW=0):  # Puissance Maximale Absorbée
        if not isinstance(start, date):
            start = parse(start).date()
        if not isinstance(end, date):
            end = parse(end).date()
        
        self.start = start
        self.end = end
        self.kWh_pointe = kWh_pointe
        self.kWh_pleine = kWh_pleine
        self.kWh_jour = kWh_jour
        self.kWh_nuit = kWh_nuit
        self.kWh_hors_pointe = kWh_hors_pointe
        self.kWh_poste_unique = kWh_poste_unique
        self.kvarh_reactif = kvarh_reactif
        self.PMA_kW = PMA_kW

class input_Contrat:
    def __init__(self, code_tarif="41", PMD_kW=0):
        self.code_tarif = code_tarif
        self.PMD_kW = PMD_kW
        # La tension sera déterminée automatiquement dans get_coefficients

class Sonalgaz_Elec:
    def __init__(self, contrat, facture):
        self.contrat = contrat
        self.facture = facture
        self.coefficients = None
        self.nb_jours = (self.facture.end - self.facture.start).days + 1
        
        # Montants calculés
        self.montant_fixe = 0
        self.montant_PMD = 0
        self.montant_PMA = 0
        self.montant_energie = 0
        self.montant_reactif = 0
        self.montant_total = 0
        self.montant_tva = 0
        self.montant_total_ttc = 0
        
        # DataFrame pour le résumé
        self.df = pd.DataFrame(columns=["Composante", "Montant (DA)"])

    def get_coefficients(self):
        """Récupère les coefficients selon le code tarif."""
        with open(coefficients_file_path, 'r') as f:
            data = json.load(f)
            
        for coef in data["coefficients"]:
            if (coef["code_tarif"] == self.contrat.code_tarif and
                parse(coef["date_debut"]).date() <= self.facture.start <= parse(coef["date_fin"]).date()):
                # Détermine la tension à partir du code tarif trouvé
                self.contrat.tension = coef["tension"]
                self.coefficients = coef  # Stocke les coefficients
                return coef
                
        raise ValueError(f"Aucun coefficient trouvé pour le code tarif {self.contrat.code_tarif}")

    def calculate(self):
        """Calcule tous les éléments de la facture selon le niveau de tension."""
        coef = self.get_coefficients()
        tension = coef["tension"]  # Récupère la tension depuis les coefficients
        
        if tension == "BT":
            self._calculate_BT(coef)
        elif tension in ["HTA", "HTB"]:
            self._calculate_HT(tension, coef)
        else:
            raise ValueError(f"Niveau de tension non reconnu : {tension}")

    def _calculate_BT(self, coef):
        """Calcul spécifique pour la Basse Tension."""
        # 1. Redevance Fixe (trimestrielle)
        self.montant_fixe = coef["fixe_DA_mois"] * 3  # Trimestre

        # 2. Facturation PMD
        self.montant_PMD = coef["souscription_DA_kW_mois"] * self.contrat.PMD_kW * 3  # Trimestre

        # 3. Energie Active selon code tarif
        if self.contrat.code_tarif in ["51M", "51NM", "52M", "52NM", "53M", "53NM"]:
            # Calcul par poste horaire
            self.montant_energie = (
                coef["point_cDA_kWh"]/100 * self.facture.kWh_pointe +
                coef["Pleine_cDA_kWh"]/100 * self.facture.kWh_pleine +
                coef["jour_cDA_kWh"]/100 * self.facture.kWh_jour +
                coef["hors_pointe_cDA_kWh"]/100 * self.facture.kWh_hors_pointe +
                coef["Nuit_cDA_kWh"]/100 * self.facture.kWh_nuit +
                coef["Poste_unique_cDA_kWh"]/100 * self.facture.kWh_poste_unique
            )
        elif self.contrat.code_tarif in ["54M", "54NM"]:
            # Calcul par tranches
            energie_mensuelle = self.facture.kWh_poste_unique / 3  # moyenne mensuelle
            self.montant_energie = self._calculate_tranches(energie_mensuelle, coef) * 3

        # 4. Total trimestriel
        self.montant_total = (
            self.montant_fixe +
            self.montant_PMD +
            self.montant_energie
        )

        # Mise à jour du DataFrame de résumé
        self._update_dataframe()

    def _calculate_HT(self, niveau, coef):
        """Calcul spécifique pour la Haute et Très Haute Tension (HTA/HTB)."""
        # 1. Redevance Fixe
        self.montant_fixe = coef["fixe_DA_mois"]
        
        # 2. Facturation PMD (Puissance Mise à Disposition)
        self.montant_PMD = coef["souscription_DA_kW_mois"] * self.contrat.PMD_kW
        
        # 3. Facturation PMA (Puissance Maximale Absorbée)
        self.montant_PMA = coef["absorbee_DA_kW_mois"] * self.facture.PMA_kW
        
        # 4. Energie Active selon le code tarif
        self.montant_energie = 0
        if niveau == "HTB" and self.contrat.code_tarif == "31":
            self.montant_energie = (
                coef["point_cDA_kWh"]/100 * self.facture.kWh_pointe +
                coef["Pleine_cDA_kWh"]/100 * self.facture.kWh_pleine +
                coef["Nuit_cDA_kWh"]/100 * self.facture.kWh_nuit
            )
        elif niveau == "HTB" and self.contrat.code_tarif == "32":
            self.montant_energie = coef["Poste_unique_cDA_kWh"]/100 * self.facture.kWh_poste_unique
        elif niveau == "HTA":
            self.montant_energie = (
                coef["point_cDA_kWh"]/100 * self.facture.kWh_pointe +
                coef["Pleine_cDA_kWh"]/100 * self.facture.kWh_pleine +
                coef["jour_cDA_kWh"]/100 * self.facture.kWh_jour +
                coef["hors_pointe_cDA_kWh"]/100 * self.facture.kWh_hors_pointe +
                coef["Nuit_cDA_kWh"]/100 * self.facture.kWh_nuit +
                coef["Poste_unique_cDA_kWh"]/100 * self.facture.kWh_poste_unique
            )

        # 5. Energie Réactive (uniquement pour HTA et HTB)
        energie_totale = (
            self.facture.kWh_pointe +
            self.facture.kWh_pleine +
            self.facture.kWh_jour +
            self.facture.kWh_nuit +
            self.facture.kWh_hors_pointe +
            self.facture.kWh_poste_unique
        )
        
        # Calcul du seuil de dépassement (50% de l'énergie active)
        seuil_reactif = 0.5 * energie_totale
        
        # Calcul du dépassement ou du non-dépassement
        depassement = self.facture.kvarh_reactif - seuil_reactif
        print(f"dépassement réactif : {depassement} kvarh (seuil : {seuil_reactif} kvarh)")
        
        # Application du tarif selon dépassement ou non
        if depassement > 0:
            # Malus sur le dépassement
            self.montant_reactif = depassement * coef["reactive_malus_cDA_kvarh"]/100
        else:
            # Bonus sur l'énergie réactive en dessous du seuil
            self.montant_reactif = depassement * coef["reactive_bonus_cDA_kvarh"]/100

        # 6. Total
        self.montant_total = (
            self.montant_fixe +
            self.montant_PMD +
            self.montant_PMA +
            self.montant_energie +
            self.montant_reactif
        )

        # Calcul de la TVA (19% sur l'énergie HT)
        self.montant_tva = self.montant_total * 0.19
        
        # Calcul du total TTC
        self.montant_total_ttc = self.montant_total + self.montant_tva

        # Mise à jour du DataFrame
        periode = "mois"
        self._update_dataframe(periode)

    def _update_dataframe(self, periode="mois"):
        """Mise à jour du DataFrame avec les résultats."""
        # Calcul des énergies totales en MWh/Mvarh
        energie_active_MWh = (
            self.facture.kWh_pointe +
            self.facture.kWh_pleine +
            self.facture.kWh_jour +
            self.facture.kWh_nuit +
            self.facture.kWh_hors_pointe +
            self.facture.kWh_poste_unique
        ) / 1000  # Conversion en MWh
        
        energie_reactive_Mvarh = self.facture.kvarh_reactif / 1000  # Conversion en Mvarh
        
        # Début du DataFrame avec les consommations
        composantes = [
            ("Energie Active totale (MWh)", f"{energie_active_MWh:,.2f}"),
            ("Energie Réactive totale (Mvarh)", f"{energie_reactive_Mvarh:,.2f}"),
            ("", "")  # Ligne vide pour la séparation
        ]
        
        # Reste des composantes
        composantes.extend([
            (f"Redevance Fixe (DA/{periode})", f"{self.montant_fixe:,.2f}"),
            (f"PMD-Puissance mise à disposition (DA/{periode})", f"{self.montant_PMD:,.2f}")
        ])
        
        if self.contrat.tension in ["HTA", "HTB"]:
            composantes.extend([
                (f"PMA-Puissance maximale atteinte (DA/{periode})", f"{self.montant_PMA:,.2f}")
            ])
            
            # Détail de l'énergie active selon le niveau de tension et code tarif
            coef = self.coefficients
            if self.contrat.tension == "HTB" and self.contrat.code_tarif == "31":
                for poste, kWh in [
                    ("Pointe", self.facture.kWh_pointe),
                    ("Pleine", self.facture.kWh_pleine),
                    ("Nuit", self.facture.kWh_nuit)
                ]:
                    if kWh > 0:
                        montant = coef[f'{poste.lower()}_cDA_kWh' if poste == "Nuit" 
                                     else f'{"point" if poste == "Pointe" else poste.lower()}_cDA_kWh']/100 * kWh
                        composantes.append((f"Energie {poste} (DA/mois)", f"{montant:,.2f}"))
                        
            elif self.contrat.tension == "HTB" and self.contrat.code_tarif == "32":
                if self.facture.kWh_poste_unique > 0:
                    montant = coef['Poste_unique_cDA_kWh']/100 * self.facture.kWh_poste_unique
                    composantes.append(("Energie Poste Unique (DA/mois)", f"{montant:,.2f}"))
                    
            elif self.contrat.tension == "HTA":
                # Mapping des postes horaires selon le code tarif
                postes_tarif = {
                    "41": ["Pointe", "Pleine", "Jour", "Hors Pointe", "Nuit", "Poste Unique"],
                    "42": ["Pointe", "Hors Pointe"],  # Uniquement Pointe et Hors Pointe
                    "43": ["Pointe", "Jour", "Nuit"],
                    "44": ["Poste Unique"]
                }
                
                mapping_coef = {
                    "Pointe": ("point_cDA_kWh", "kWh_pointe"),
                    "Pleine": ("Pleine_cDA_kWh", "kWh_pleine"),
                    "Jour": ("jour_cDA_kWh", "kWh_jour"),
                    "Hors Pointe": ("hors_pointe_cDA_kWh", "kWh_hors_pointe"),
                    "Nuit": ("Nuit_cDA_kWh", "kWh_nuit"),
                    "Poste Unique": ("Poste_unique_cDA_kWh", "kWh_poste_unique")
                }
                
                for poste in postes_tarif.get(self.contrat.code_tarif, []):
                    coef_name, attr_name = mapping_coef[poste]
                    kWh = getattr(self.facture, attr_name)
                    if kWh > 0:
                        montant = coef[coef_name]/100 * kWh
                        composantes.append((f"Energie {poste} (DA/mois)", f"{montant:,.2f}"))
            
            if self.montant_reactif != 0:
                composantes.extend([
                    (f"Energie Réactive (DA/{periode})", f"{self.montant_reactif:,.2f}")
                ])
        
        # Calcul du prix moyen par MWh
        energie_totale_MWh = (
            self.facture.kWh_pointe +
            self.facture.kWh_pleine +
            self.facture.kWh_jour +
            self.facture.kWh_nuit +
            self.facture.kWh_hors_pointe +
            self.facture.kWh_poste_unique
        ) / 1000  # Conversion en MWh
        
        prix_moyen_MWh = self.montant_total / energie_totale_MWh if energie_totale_MWh > 0 else 0
       
        composantes.extend([
            (f"Energie Active Total (DA/{periode})", f"{self.montant_energie:,.2f}"),
            (f"Prix moyen énergie active (DA HTVA/MWh)", f"{prix_moyen_MWh:,.2f}"),
            (f"Total HTVA (DA/{periode})", f"{self.montant_total:,.2f}"),
            (f"TVA 19% (DA/{periode})", f"{self.montant_tva:,.2f}"),
            (f"Total TTC (DA/{periode})", f"{self.montant_total_ttc:,.2f}")
        ])
        
        self.df = pd.DataFrame(composantes, columns=["Composante", "Montant (DA)"])
        pd.set_option('display.float_format', lambda x: '{:,.2f}'.format(x))

    def plot(self, title="Facture d'Électricité SONALGAZ (DA)", figsize=(8, 6)):
        """Affiche un graphique en donut des composantes principales de la facture."""
        # Séparation de l'énergie réactive en bonus/malus
        reactif_bonus = min(0, self.montant_reactif)  # Valeur négative ou 0
        reactif_malus = max(0, self.montant_reactif)  # Valeur positive ou 0
        
        # Ajuster les labels et valeurs selon le type d'énergie réactive
        if reactif_bonus < 0:
            labels = ['Fixe et PMD(Puissance mise à disposition)', 'PMA(Puissance maximale atteinte)', 'Énergie', 'Réactif (bonus)', 'TVA']
            values = [
                self.montant_fixe + self.montant_PMD,
                self.montant_PMA,
                self.montant_energie,
                abs(reactif_bonus),  # Conversion en positif pour l'affichage
                self.montant_tva
            ]
        else:
            labels = ['Fixe et PMD(Puissance mise à disposition)', 'PMA(Puissance maximale atteinte)', 'Énergie', 'Réactif (malus)', 'TVA']
            values = [
                self.montant_fixe + self.montant_PMD,
                self.montant_PMA,
                self.montant_energie,
                reactif_malus,
                self.montant_tva
            ]
        
        colors = ['#4CAF50', '#2196F3', '#F44336', '#9C27B0', '#FFC107']
        
        total = self.montant_total_ttc
        
        fig, ax = plt.subplots(figsize=figsize)
        
        wedges, texts, autotexts = ax.pie(
            values,
            labels=None,
            autopct='%1.1f%%',
            startangle=90,
            colors=colors,
            wedgeprops=dict(width=0.4, edgecolor='w'),
            pctdistance=0.8,
            textprops={'fontsize': 12}
        )
        
        ax.axis('equal')
        plt.title(title, fontsize=12, pad=16)
        
        # Ajout du total au centre du donut
        ax.text(0, 0, f"{total:,.2f} DA", ha='center', va='center', fontsize=14, fontweight='bold')
        
        # Légende en bas à gauche avec indication bonus/malus
        y_start = -0.05
        y_step = 0.07
        x_rect = -0.12
        x_text = -0.05
        
        for i, (label, val, color) in enumerate(zip(labels, values, colors)):
            y = y_start - i * y_step
            rect = Rectangle((x_rect, y - 0.02), 0.04, 0.04, facecolor=color, transform=ax.transAxes, clip_on=False)
            ax.add_patch(rect)
            # Affichage du signe négatif pour le bonus
            if "bonus" in label:
                ax.text(x_text, y, f"{label} : -{val:,.2f} DA",
                       transform=ax.transAxes,
                       ha='left', va='center',
                       fontsize=11, weight='bold')
            else:
                ax.text(x_text, y, f"{label} : {val:,.2f} DA",
                       transform=ax.transAxes,
                       ha='left', va='center',
                       fontsize=11, weight='bold')
        
        plt.tight_layout()
        plt.show()

    def plot_detail(self, title="Détail des Composantes de la Facture SONALGAZ", figsize=(16, 6)):
        """Affiche un graphique en cascade (waterfall) des composantes détaillées."""
        fig, axs = plt.subplots(1, 2, figsize=figsize)  # Changé de 3 à 2 graphiques

        def plot_waterfall(ax, labels, values, title):
            cum_values = [0]
            for v in values:
                cum_values.append(cum_values[-1] + v)

            colors = ['green' if v >= 0 else 'red' for v in values]
            for i, (label, val) in enumerate(zip(labels, values)):
                ax.bar(i, val, bottom=cum_values[i], color=colors[i], edgecolor='black')
                y_text = cum_values[i] + val / 2
                ax.text(i, y_text, f"{val:.2f} DA", ha='center', va='center', fontsize=9,
                        color='black' if abs(val) > 20 else 'black')

            total = sum(values)
            ax.set_xticks(range(len(labels)))
            ax.set_xticklabels(labels, rotation=45, ha='right')
            ax.axhline(0, color='black', linewidth=0.8)
            ax.set_title(f"{title}\nTotal : {total:,.2f} DA", fontsize=12)
            ax.grid(axis='y', linestyle='--', alpha=0.5)

        # Composantes fixes
        fixes_labels = ['Redevance Fixe', 'PMD-Puissance mise à disposition', 'PMA-Puissance maximale atteinte']
        fixes_values = [
            self.montant_fixe,
            self.montant_PMD,
            self.montant_PMA
        ]
        plot_waterfall(axs[0], fixes_labels, fixes_values, "Composantes Fixes")

        # Énergie active et réactive
        energie_labels = []
        energie_values = []
        
        # Ajout des composantes d'énergie active selon le code tarif
        if self.contrat.code_tarif == "42":
            energie_labels.extend(['Pointe', 'Hors Pointe'])
            energie_values.extend([
                self.coefficients['point_cDA_kWh']/100 * self.facture.kWh_pointe,
                self.coefficients['hors_pointe_cDA_kWh']/100 * self.facture.kWh_hors_pointe
            ])
        else:
            energie_labels.extend(['Pointe', 'Pleine', 'Jour', 'Nuit', 'Hors Pointe'])
            energie_values.extend([
                self.coefficients['point_cDA_kWh']/100 * self.facture.kWh_pointe,
                self.coefficients['Pleine_cDA_kWh']/100 * self.facture.kWh_pleine,
                self.coefficients['jour_cDA_kWh']/100 * self.facture.kWh_jour,
                self.coefficients['Nuit_cDA_kWh']/100 * self.facture.kWh_nuit,
                self.coefficients['hors_pointe_cDA_kWh']/100 * self.facture.kWh_hors_pointe
            ])
        
        # Ajout de l'énergie réactive
        energie_labels.append('Réactive')
        energie_values.append(self.montant_reactif)
        
        plot_waterfall(axs[1], energie_labels, energie_values, "Énergies Active et Réactive")

        plt.suptitle(title, fontsize=16, y=0.97)
        plt.tight_layout(rect=[0, 0, 1, 0.93])
        plt.show()


from ThermodynamicCycles.FluidPort.FluidPort import FluidPort
from CoolProp.CoolProp import PropsSI
import pandas as pd 

class Object:
    def __init__(self):
        self.Timestamp=None

        #Input and Output Connector
        self.Inlet=FluidPort() 
        self.Outlet=FluidPort()

        #Input Data
        self.eta_is=0.75
        self.HP_bar=None
        self.Tcond_degC=None
        self.HP=None #self.HP_bar*100000
        self.Tdischarge_target=None #°C
        

        #Initial Values
        self.Inlet.fluid="air"
        self.Inlet.P=None
        self.F=0.1
        self.Inlet.F=self.F

        self.F_Sm3s=None
        self.F_Sm3h=None
        
        #Output Data
        self.df=[]
        self.HeatLossesRatio=0
        self.So_is=0
        self.To_is=0
        self.Ho_is=0
        self.Ho_ref=0
        self.To_ref=0
        self.So_ref=0
        self.To=0
        self.Q_comp=None
        self.Q_losses=0
        self.Ti_degC=0
        
    def calculate (self):


        if self.Tcond_degC is not None:
            self.HP = PropsSI('P','T',self.Tcond_degC+273.15,'Q',1,self.Inlet.fluid)
            print("test HP calculation",self.HP)

        if self.HP_bar is not None:
            self.HP=self.HP_bar*100000
        if self.HP is not None:
            self.HP_bar=self.HP/100000
        
        #Inlet temperature calculation
        self.Ti_degC=PropsSI('T','P',self.Inlet.P,'H',self.Inlet.h,self.Inlet.fluid)-273.15
        
        self.So_is = PropsSI('S','P',self.Inlet.P,'H',self.Inlet.h,self.Inlet.fluid)
        self.To_is=PropsSI('T','P',self.HP,'S',self.So_is,self.Inlet.fluid)
        self.Ho_is = PropsSI('H','P',self.HP,'S',self.So_is,self.Inlet.fluid)
        
        self.Ho_ref = (self.Ho_is-self.Inlet.h)/self.eta_is+self.Inlet.h
        self.To_ref=PropsSI('T','P',self.HP,'H',self.Ho_ref,self.Inlet.fluid)
        self.So_ref=PropsSI('S','P',self.HP,'H',self.Ho_ref,self.Inlet.fluid)
        
               
        if self.Tdischarge_target<=(self.To_ref-273.15):
            self.To=self.Tdischarge_target+273.15
            print("Le compresseur est refroidi")

        if self.Tdischarge_target>(self.To_ref-273.15):  
            self.Tdischarge_target=self.To_ref-273.15    # Passer un compresseur adiabatique
            self.To=self.To_ref
            print("Le compresseur n'est pas refroidi")
        
        self.Ho=PropsSI('H','P',self.HP,'T',self.To,self.Inlet.fluid)
        self.So=PropsSI('S','P',self.HP,'T',self.To,self.Inlet.fluid)
        
        #mass flow rate or Energy Power Calculation

        if self.Q_comp is not None:
            self.F=self.Q_comp/(self.Ho_ref-self.Inlet.h)
        else: # self.Inlet.F is not None:
            self.F=self.Inlet.F
            self.Q_comp=self.F*(self.Ho_ref-self.Inlet.h)
        
        

        self.Q_losses=self.F*(self.Ho_ref-self.Ho)

        self.HeatLossesRatio=self.Q_losses/self.Q_comp

        self.F_Sm3s=self.F/PropsSI("D", "P", 100000*1.01325, "T", (15+273.15), self.Inlet.fluid)
        self.F_Sm3h=self.F_Sm3s*3600

        self.eta_WhSm3=self.Q_comp/self.F_Sm3h
        
        # outlet connector calculation
        self.Outlet.fluid=self.Inlet.fluid
        self.Outlet.h=self.Ho
        self.Outlet.F=self.F
        self.Outlet.P=self.HP

        # Results
        self.df = pd.DataFrame({'Compressor': [self.Timestamp,self.Inlet.fluid,self.F,self.Q_comp/1000, self.Q_losses/1000, self.HeatLossesRatio,self.To_is-273.15,self.eta_WhSm3,self.To_is-273.15,self.Ho_is/1000,self.So_is/1000,self.To_ref-273.15,self.Ho_ref/1000,self.So_ref/1000,self.To-273.15,self.Ho/1000,self.So/1000,self.Outlet.P/100000,], },
                      index = ['Timestamp','comp_fluid','comp_F_kgs','Q_comp(KW)', 'Q_losses(KW)', 'HeatLossesRatio','Tis(°C)','eta_WhSm3',"To_is(°C)","H3is(kJ/kg)","S3is (J/kg-K)","T3ref(°C)","H3ref(kJ/kg)", "S3ref(J/kg-K)",
        "To(°C)", "Ho(kJ/kg)", "So(J/kg-K)","self.Outlet.P (bar)"])

      
       
        
        


# -*- coding: utf-8 -*-
"""
A module containing NodeEditor's class for representing Edge and Edge Type Constants.
"""
from collections import OrderedDict
from NodeEditor.nodeeditor.node_graphics_edge import *
from NodeEditor.nodeeditor.node_serializable import Serializable
from NodeEditor.nodeeditor.utils import dumpException


EDGE_TYPE_DIRECT = 1        #:
EDGE_TYPE_BEZIER = 2        #:

DEBUG = False


class Edge(Serializable):
    """
    Class for representing Edge in NodeEditor.
    """

    edge_validators = []        #: class variable containing list of registered edge validators

    def __init__(self, scene:'Scene', start_socket:'Socket'=None, end_socket:'Socket'=None, edge_type=EDGE_TYPE_DIRECT):
        """

        :param scene: Reference to the :py:class:`~nodeeditor.node_scene.Scene`
        :type scene: :py:class:`~nodeeditor.node_scene.Scene`
        :param start_socket: Reference to the starting socket
        :type start_socket: :py:class:`~nodeeditor.node_socket.Socket`
        :param end_socket: Reference to the End socket or ``None``
        :type end_socket: :py:class:`~nodeeditor.node_socket.Socket` or ``None``
        :param edge_type: Constant determining type of edge. See :ref:`edge-type-constants`

        :Instance Attributes:

            - **scene** - reference to the :class:`~nodeeditor.node_scene.Scene`
            - **grEdge** - Instance of :class:`~nodeeditor.node_graphics_edge.QDMGraphicsEdge` subclass handling graphical representation in the ``QGraphicsScene``.
        """
        super().__init__()
        self.scene = scene

        # default init
        self._start_socket = None
        self._end_socket = None

        self.start_socket = start_socket
        self.end_socket = end_socket
        self._edge_type = edge_type

        # create Graphics Edge instance
        self.grEdge = self.createEdgeClassInstance()

        self.scene.addEdge(self)

    def __str__(self):
        return "<Edge %s..%s -- S:%s E:%s>" % (
            hex(id(self))[2:5], hex(id(self))[-3:],
            self.start_socket, self.end_socket
        )
    @property
    def start_socket(self):
        """
        Start socket

        :getter: Returns start :class:`~nodeeditor.node_socket.Socket`
        :setter: Sets start :class:`~nodeeditor.node_socket.Socket` safely
        :type: :class:`~nodeeditor.node_socket.Socket`
        """
        return self._start_socket

    @start_socket.setter
    def start_socket(self, value):
        # if we were assigned to some socket before, delete us from the socket
        if self._start_socket is not None:
            self._start_socket.removeEdge(self)

        # assign new start socket
        self._start_socket = value
        # addEdge to the Socket class
        if self.start_socket is not None:
            self.start_socket.addEdge(self)

    @property
    def end_socket(self):
        """
        End socket

        :getter: Returns end :class:`~nodeeditor.node_socket.Socket` or ``None`` if not set
        :setter: Sets end :class:`~nodeeditor.node_socket.Socket` safely
        :type: :class:`~nodeeditor.node_socket.Socket` or ``None``
        """
        return self._end_socket

    @end_socket.setter
    def end_socket(self, value):
        # if we were assigned to some socket before, delete us from the socket
        if self._end_socket is not None:
            self._end_socket.removeEdge(self)

        # assign new end socket
        self._end_socket= value
        # addEdge to the Socket class
        if self.end_socket is not None:
            self.end_socket.addEdge(self)

    @property
    def edge_type(self):
        """
        Edge type

        :getter: get edge type constant for current ``Edge``. See :ref:`edge-type-constants`
        :setter: sets new edge type. On background, creates new :class:`~nodeeditor.node_graphics_edge.QDMGraphicsEdge`
            child class if necessary, adds this ``QGraphicsPathItem`` to the ``QGraphicsScene`` and updates edge sockets
            positions.
        """
        return self._edge_type

    @edge_type.setter
    def edge_type(self, value):
        # assign new value
        self._edge_type = value

        # update the grEdge pathCalculator
        self.grEdge.createEdgePathCalculator()

        if self.start_socket is not None:
            self.updatePositions()

    @classmethod
    def getEdgeValidators(cls):
        """Return the list of Edge Validator Callbacks"""
        return cls.edge_validators

    @classmethod
    def registerEdgeValidator(cls, validator_callback: 'function'):
        """Register Edge Validator Callback

        :param validator_callback: A function handle to validate Edge
        :type validator_callback: `function`
        """
        cls.edge_validators.append(validator_callback)

    @classmethod
    def validateEdge(cls, start_socket: 'Socket', end_socket: 'Socket') -> bool:
        """Validate Edge agains all registered `Edge Validator Callbacks`

        :param start_socket: Starting :class:`~nodeeditor.node_socket.Socket` of Edge to check
        :type start_socket: :class:`~nodeeditor.node_socket.Socket`
        :param end_socket: Target/End :class:`~nodeeditor.node_socket.Socket` of Edge to check
        :type end_socket: :class:`~nodeeditor.node_socket.Socket`
        :return: ``True`` if the Edge is valid or ``False`` if not
        :rtype: ``bool``
        """
        for validator in cls.getEdgeValidators():
            if not validator(start_socket, end_socket):
                return False
        return True

    def reconnect(self, from_socket: 'Socket', to_socket: 'Socket'):
        """Helper function which reconnects edge `from_socket` to `to_socket`"""
        if self.start_socket == from_socket:
            self.start_socket = to_socket
        elif self.end_socket == from_socket:
            self.end_socket = to_socket

    def getGraphicsEdgeClass(self):
        """Returns the class representing Graphics Edge"""
        return QDMGraphicsEdge

    def createEdgeClassInstance(self):
        """
        Create instance of grEdge class
        :return: Instance of `grEdge` class representing the Graphics Edge in the grScene
        """
        self.grEdge = self.getGraphicsEdgeClass()(self)
        self.scene.grScene.addItem(self.grEdge)
        if self.start_socket is not None:
            self.updatePositions()
        return self.grEdge

    def getOtherSocket(self, known_socket:'Socket'):
        """
        Returns the oposite socket on this ``Edge``

        :param known_socket: Provide known :class:`~nodeeditor.node_socket.Socket` to be able to determine the oposite one.
        :type known_socket: :class:`~nodeeditor.node_socket.Socket`
        :return: The oposite socket on this ``Edge`` or ``None``
        :rtype: :class:`~nodeeditor.node_socket.Socket` or ``None``
        """
        return self.start_socket if known_socket == self.end_socket else self.end_socket

    def doSelect(self, new_state:bool=True):
        """
        Provide the safe selecting/deselecting operation. In the background it takes care about the flags, norifications
        and storing history for undo/redo.

        :param new_state: ``True`` if you want to select the ``Edge``, ``False`` if you want to deselect the ``Edge``
        :type new_state: ``bool``
        """
        self.grEdge.doSelect(new_state)

    def updatePositions(self):
        """
        Updates the internal `Graphics Edge` positions according to the start and end :class:`~nodeeditor.node_socket.Socket`.
        This should be called if you update ``Edge`` positions.
        """
        source_pos = self.start_socket.getSocketPosition()
        source_pos[0] += self.start_socket.node.grNode.pos().x()
        source_pos[1] += self.start_socket.node.grNode.pos().y()
        self.grEdge.setSource(*source_pos)
        if self.end_socket is not None:
            end_pos = self.end_socket.getSocketPosition()
            end_pos[0] += self.end_socket.node.grNode.pos().x()
            end_pos[1] += self.end_socket.node.grNode.pos().y()
            self.grEdge.setDestination(*end_pos)
        else:
            self.grEdge.setDestination(*source_pos)
        self.grEdge.update()


    def remove_from_sockets(self):
        """
        Helper function which sets start and end :class:`~nodeeditor.node_socket.Socket` to ``None``
        """
        self.end_socket = None
        self.start_socket = None


    def remove(self, silent_for_socket:'Socket'=None, silent=False):
        """
        Safely remove this Edge.

        Removes `Graphics Edge` from the ``QGraphicsScene`` and it's reference to all GC to clean it up.
        Notifies nodes previously connected :class:`~nodeeditor.node_node.Node` (s) about this event.

        Triggers Nodes':

        - :py:meth:`~nodeeditor.node_node.Node.onEdgeConnectionChanged`
        - :py:meth:`~nodeeditor.node_node.Node.onInputChanged`

        :param silent_for_socket: :class:`~nodeeditor.node_socket.Socket` of a :class:`~nodeeditor.node_node.Node` which
            won't be notified, when this ``Edge`` is going to be removed
        :type silent_for_socket: :class:`~nodeeditor.node_socket.Socket`
        :param silent: ``True`` if no events should be triggered during removing
        :type silent: ``bool``
        """
        old_sockets = [self.start_socket, self.end_socket]

        # ugly hack, since I noticed that even when you remove grEdge from scene,
        # sometimes it stays there! How dare you Qt!
        if DEBUG: print(" - hide grEdge")
        self.grEdge.hide()

        if DEBUG: print(" - remove grEdge", self.grEdge)
        self.scene.grScene.removeItem(self.grEdge)
        if DEBUG: print("   grEdge:", self.grEdge)

        self.scene.grScene.update()

        if DEBUG: print("# Removing Edge", self)
        if DEBUG: print(" - remove edge from all sockets")
        self.remove_from_sockets()
        if DEBUG: print(" - remove edge from scene")
        try:
            self.scene.removeEdge(self)
        except ValueError:
            pass
        if DEBUG: print(" - everything is done.")

        try:
            # notify nodes from old sockets
            for socket in old_sockets:
                if socket and socket.node:
                    if silent:
                        continue
                    if silent_for_socket is not None and socket == silent_for_socket:
                        # if we requested silence for Socket and it's this one, skip notifications
                        continue

                    # notify Socket's Node
                    socket.node.onEdgeConnectionChanged(self)
                    if socket.is_input: socket.node.onInputChanged(socket)

        except Exception as e: dumpException(e)


    def serialize(self) -> OrderedDict:
        return OrderedDict([
            ('id', self.id),
            ('edge_type', self.edge_type),
            ('start', self.start_socket.id if self.start_socket is not None else None),
            ('end', self.end_socket.id if self.end_socket is not None else None),
        ])

    def deserialize(self, data:dict, hashmap:dict={}, restore_id:bool=True, *args, **kwargs) -> bool:
        if restore_id: self.id = data['id']
        self.start_socket = hashmap[data['start']]
        self.end_socket = hashmap[data['end']]
        self.edge_type = data['edge_type']


# Example: using validators for Edge
# You can register edge validators wherever you want, even here...
# However if you do use overriden Edge, you should call registerEdgeValidator on that overriden class
#
# from nodeeditor.node_edge_validators import *
# Edge.registerEdgeValidator(edge_validator_debug)
# Edge.registerEdgeValidator(edge_cannot_connect_two_outputs_or_two_inputs)
# Edge.registerEdgeValidator(edge_cannot_connect_input_and_output_of_same_node)


# -*- coding: utf-8 -*-
"""
Created on Sat Sep  5 19:39:26 2020

@author: zohei
"""

from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.utils import dumpException

#############les modèles d'un groupe frigorifique#################
#from Modules_GroupeFrigorifique.Evaporator import Evaporator
#from Modules_GroupeFrigorifique.Compressor import Compressor
#from Modules_GroupeFrigorifique.Desuperheater import Desuperheater
#from Modules_GroupeFrigorifique.Expansion_Valve import Expansion_Valve
from ThermodynamicCycles.Condenser import Condenser
from ThermodynamicCycles.Connect import Fluid_connect
from ThermodynamicCycles.FluidPort.FluidPort import FluidPort




class CalcCondContent(QDMNodeContentWidget):
    def initUI(self):
        
        self.subcooling_lbl=QLabel("subcooling", self) 
        self.subcooling_edit=QLineEdit("0.01", self)
        
        # self.IsenEff_lbl=QLabel("Rendement isentropique", self) 
        # self.IsenEff_edit=QLineEdit("0.7", self)
        
        
        
        # self.Tref_lbl=QLabel("refoidissement cible (°C)", self) 
        # self.Tref_edit=QLineEdit("80", self)
        
        self.Tcond_lbl_title = QLabel("Tcond(°C)", self)
        self.Tcond_lbl = QLabel("", self)
        
        self.Qcond_lbl_title = QLabel("Q_cond (kW):", self)
        self.Qcond_lbl = QLabel("", self)
        
        # self.Tis_lbl_title = QLabel("Temp. isentrop. (°C)", self)
        # self.Tis_lbl = QLabel("", self)
        
        
        
        
         
        self.layout=QVBoxLayout()

        self.layout.addWidget(self.subcooling_lbl)
        self.layout.addWidget(self.subcooling_edit) 
        
        # self.layout.addWidget(self.Tref_lbl)
        # self.layout.addWidget(self.Tref_edit)
        
        # self.layout.addWidget(self.IsenEff_lbl)
        # self.layout.addWidget(self.IsenEff_edit)
        
        
        
        self.layout.addWidget(self.Tcond_lbl_title)
        self.layout.addWidget(self.Tcond_lbl)  
        
        self.layout.addWidget(self.Qcond_lbl_title)
        self.layout.addWidget(self.Qcond_lbl) 
      
        # self.layout.addWidget(self.Tis_lbl_title)
        # self.layout.addWidget(self.Tis_lbl)
        
     
        
                         
        
        self.setLayout(self.layout)
        
        self.layout.setAlignment(Qt.AlignRight)
        self.layout.setObjectName(self.node.content_label_objname)
        
        
    def serialize(self):
        res = super().serialize()
        res['value'] = self.subcooling_edit.text()
        # res2 = super().serialize()
        # res2['value2'] = self.P_edit.text()
        
        # res3 = super().serialize()
        # res3['value3'] = self.fluid.currentText()
        
        # res4 = super().serialize()
        # res4['value4'] = self.F_kgs_edit.text()
        
        return res #,res2,res3,res4

    def deserialize(self, data, hashmap={}):
        res = super().deserialize(data, hashmap)
        # res2 = super().deserialize(data, hashmap)
        # res3 = super().deserialize(data, hashmap)
        # res4 = super().deserialize(data, hashmap)
        # print("res=",res,res2,res3,res4)
        # print("dataaaaaaaaaa=",data)
        try:
            
            value = data["value"]
            # value2 = data[1]['value2']
            # value3 = data[2]['value3']
            # value4 = data[3]['value4']
            
            # print("values=",value,value2,value3,value4)
            
            self.subcooling_edit.setText(value)
            # self.P_edit.setText(value2)
            # self.fluid.setCurrentText(value3)
            # self.F_kgs_edit.setText(value4)
            
            return True & res # & res2 & res3 & res4
        except Exception as e:
            dumpException(e)
        return res #,res2,res3,res4


@register_node(OP_NODE_COND)
class CalcNode_Cond(CalcNode):
    icon = "icons/condenser.png"
    op_code = OP_NODE_COND
    op_title = "Condenseur"
    content_label = "/"
    content_label_objname = "calc_node_cond"
    
    def __init__(self, scene):
        super().__init__(scene, inputs=[2], outputs=[1])
        self.eval()
        
    def initInnerClasses(self):
        self.content = CalcCondContent(self)
        self.grNode = CalcGraphicsNode(self)
        
        
        self.grNode.height=210
        self.grNode.width=200
        
        self.content.subcooling_edit.textChanged.connect(self.onInputChanged)
        # self.content.Tref_edit.textChanged.connect(self.onInputChanged)
        # self.content.IsenEff_edit.textChanged.connect(self.onInputChanged)
  
    def evalOperation(self, input1, input2):
        self.value=[]
        
        a=FluidPort()
        a.fluid=input1[0]
        a.F=input1[1]
        a.P=input1[2]*1e5
        a.h=input1[3]*1000
       
        # COMP=Compressor.Compressor()
        # connect(COMP.Inlet,a)
        # ################""""
        # u_HP = self.content.HP_edit.text()
        # s_HP = float(u_HP)
        # ####################
        # COMP.HP=1e5*s_HP
        # COMP.IsenEff=float(self.content.IsenEff_edit.text()) 
        # COMP.Tdischarge_target=float(self.content.Tref_edit.text())
        # COMP.calculate()
        
                #####Condenseur    
      
        COND=Condenser.Object()
        Fluid_connect(COND.Inlet,a)
        COND.subcooling=float(self.content.subcooling_edit.text()) 
        COND.calculate()
        
       
        self.value.append(COND.Outlet.fluid) #fluide
        self.value.append(COND.Outlet.F) #débit
        self.value.append(COND.Outlet.P/1e5) #pression min
        self.value.append(COND.Outlet.h/1000) #Enthalpie
        
        # self.content.fluid_lbl.setText("%f" % (COMP.Q_comp/1000)) #"%d" % 
        self.content.Qcond_lbl.setText("%f" % (COND.Q_cond/1000))
        self.content.Tcond_lbl.setText("%f" % (COND.Tl_sat-273.15))
        
        #self.content.lbl.setText("%f" % val[3])
        
        
            
        return self.value


from ThermodynamicCycles.FluidPort.FluidPort import FluidPort
from CoolProp.CoolProp import PropsSI

class Object:
    def __init__(self):
        self.IsenEff=0.7
        self.Inlet=FluidPort() 
        self.F=0.1
        self.Inlet.F=self.F
        self.Outlet=FluidPort()
        self.So_is=0
        self.To_is=0
        self.Ho_is=0
        self.LP=1*100000
        self.Ho=0
        self.To=0
        self.So=0
   
        self.Q_turb=0
   
        
    def calculate (self):
        self.F=self.Inlet.F
        self.So_is = PropsSI('S','P',self.Inlet.P,'H',self.Inlet.h,self.Inlet.fluid)
        self.To_is=PropsSI('T','P',self.LP,'S',self.So_is,self.Inlet.fluid)
        self.Ho_is = PropsSI('H','P',self.LP,'S',self.So_is,self.Inlet.fluid)
        
        self.Ho = (self.Ho_is-self.Inlet.h)*self.IsenEff+self.Inlet.h
        self.To=PropsSI('T','P',self.LP,'H',self.Ho,self.Inlet.fluid)
        self.So=PropsSI('S','P',self.LP,'H',self.Ho,self.Inlet.fluid)      
        
        self.Outlet.fluid=self.Inlet.fluid
        self.Outlet.h=self.Ho
        self.Outlet.F=self.F
        self.Outlet.P=self.LP
        
        self.Q_turb=-self.Inlet.F*(self.Ho-self.Inlet.h)
        
        


# -*- coding: utf-8 -*-
"""
A module containing all code for working with History (Undo/Redo)
"""
from NodeEditor.nodeeditor.node_graphics_edge import QDMGraphicsEdge
from NodeEditor.nodeeditor.utils import dumpException

DEBUG = False
DEBUG_SELECTION = False


class SceneHistory():
    """Class contains all the code for undo/redo operations"""
    def __init__(self, scene:'Scene'):
        """
        :param scene: Reference to the :class:`~nodeeditor.node_scene.Scene`
        :type scene: :class:`~nodeeditor.node_scene.Scene`

        :Instance Attributes:

        - **scene** - reference to the :class:`~nodeeditor.node_scene.Scene`
        - **history_limit** - number of history steps that can be stored
        """
        self.scene = scene

        self.clear()
        self.history_limit = 32

        self.undo_selection_has_changed = False

        # listeners
        self._history_modified_listeners = []
        self._history_stored_listeners = []
        self._history_restored_listeners = []

    def clear(self):
        """Reset the history stack"""
        self.history_stack = []
        self.history_current_step = -1

    def storeInitialHistoryStamp(self):
        """Helper function usually used when new or open file requested"""
        self.storeHistory("Initial History Stamp")

    def addHistoryModifiedListener(self, callback:'function'):
        """
        Register callback for `HistoryModified` event

        :param callback: callback function
        """
        self._history_modified_listeners.append(callback)

    def addHistoryStoredListener(self, callback:'function'):
        """
        Register callback for `HistoryStored` event

        :param callback: callback function
        """
        self._history_stored_listeners.append(callback)

    def addHistoryRestoredListener(self, callback:'function'):
        """
        Register callback for `HistoryRestored` event

        :param callback: callback function
        """
        self._history_restored_listeners.append(callback)

    def canUndo(self) -> bool:
        """Return ``True`` if Undo is available for current `History Stack`

        :rtype: ``bool``
        """
        return self.history_current_step > 0

    def canRedo(self) -> bool:
        """
        Return ``True`` if Redo is available for current `History Stack`

        :rtype: ``bool``
        """
        return self.history_current_step + 1 < len(self.history_stack)

    def undo(self):
        """Undo operation"""
        if DEBUG: print("UNDO")

        if self.canUndo():
            self.history_current_step -= 1
            self.restoreHistory()
            self.scene.has_been_modified = True

    def redo(self):
        """Redo operation"""
        if DEBUG: print("REDO")
        if self.canRedo():
            self.history_current_step += 1
            self.restoreHistory()
            self.scene.has_been_modified = True


    def restoreHistory(self):
        """
        Restore `History Stamp` from `History stack`.

        Triggers:

        - `History Modified` event
        - `History Restored` event
        """
        if DEBUG: print("Restoring history",
                        ".... current_step: @%d" % self.history_current_step,
                        "(%d)" % len(self.history_stack))
        self.restoreHistoryStamp(self.history_stack[self.history_current_step])
        for callback in self._history_modified_listeners: callback()
        for callback in self._history_restored_listeners: callback()


    def storeHistory(self, desc:str, setModified:bool=False):
        """
        Store History Stamp into History Stack

        :param desc: Description of current History Stamp
        :type desc: ``str``
        :param setModified: if ``True`` marks :class:`~nodeeditor.node_scene.Scene` with `has_been_modified`
        :type setModified: ``bool``

        Triggers:

        - `History Modified`
        - `History Stored`
        """
        if setModified:
            self.scene.has_been_modified = True

        if DEBUG: print("Storing history", '"%s"' % desc,
                        ".... current_step: @%d" % self.history_current_step,
                        "(%d)" % len(self.history_stack))

        # if the pointer (history_current_step) is not at the end of history_stack
        if self.history_current_step+1 < len(self.history_stack):
            self.history_stack = self.history_stack[0:self.history_current_step+1]

        # history is outside of the limits
        if self.history_current_step+1 >= self.history_limit:
            self.history_stack = self.history_stack[1:]
            self.history_current_step -= 1

        hs = self.createHistoryStamp(desc)

        self.history_stack.append(hs)
        self.history_current_step += 1
        if DEBUG: print("  -- setting step to:", self.history_current_step)

        # always trigger history modified (for i.e. updateEditMenu)
        for callback in self._history_modified_listeners: callback()
        for callback in self._history_stored_listeners: callback()


    def captureCurrentSelection(self) -> dict:
        """
        Create dictionary with list of selected nodes and list of selected edges
        :return: ``dict`` 'nodes' - list of selected nodes, 'edges' - list of selected edges
        :rtype: ``dict``
        """
        sel_obj = {
            'nodes': [],
            'edges': [],
        }
        for item in self.scene.grScene.selectedItems():
            if hasattr(item, 'node'): sel_obj['nodes'].append(item.node.id)
            elif hasattr(item, 'edge'): sel_obj['edges'].append(item.edge.id)
        return sel_obj

    def createHistoryStamp(self, desc:str) -> dict:
        """
        Create History Stamp. Internally serialize whole scene and current selection

        :param desc: Descriptive label for the History Stamp
        :return: History stamp serializing state of `Scene` and current selection
        :rtype: ``dict``
        """
        history_stamp = {
            'desc': desc,
            'snapshot': self.scene.serialize(),
            'selection': self.captureCurrentSelection(),
        }

        return history_stamp

    def restoreHistoryStamp(self, history_stamp:dict):
        """
        Restore History Stamp to current `Scene` with selection of items included

        :param history_stamp: History Stamp to restore
        :type history_stamp: ``dict``
        """
        if DEBUG: print("RHS: ", history_stamp['desc'])

        try:
            self.undo_selection_has_changed = False
            previous_selection = self.captureCurrentSelection()
            if DEBUG_SELECTION: print("selected nodes before restore:", previous_selection['nodes'])

            self.scene.deserialize(history_stamp['snapshot'])

            # restore selection

            # first clear all selection on edges
            for edge in self.scene.edges: edge.grEdge.setSelected(False)
            # now restore selected edges from history_stamp
            for edge_id in history_stamp['selection']['edges']:
                for edge in self.scene.edges:
                    if edge.id == edge_id:
                        edge.grEdge.setSelected(True)
                        break

            # first clear all selection on nodes
            for node in self.scene.nodes: node.grNode.setSelected(False)
            # now restore selected nodes from history_stamp
            for node_id in history_stamp['selection']['nodes']:
                for node in self.scene.nodes:
                    if node.id == node_id:
                        node.grNode.setSelected(True)
                        break

            current_selection = self.captureCurrentSelection()
            if DEBUG_SELECTION: print("selected nodes after restore:", current_selection['nodes'])

            # reset the last_selected_items - since we're comparing change to the last_selected state
            self.scene._last_selected_items = self.scene.getSelectedItems()

            # if the selection of nodes differ before and after restoration, set flag
            if current_selection['nodes'] != previous_selection['nodes'] or current_selection['edges'] != previous_selection['edges']:
                if DEBUG_SELECTION: print("\nSCENE: Selection has changed")
                self.undo_selection_has_changed = True

        except Exception as e: dumpException(e)


from ThermodynamicCycles.FluidPort.FluidPort import FluidPort
from CoolProp.CoolProp import PropsSI
import pandas as pd 
from datetime import datetime

import math

class Object:
    def __init__(self):
        self.Timestamp=datetime.now().strftime("%Y-%m-%d %H:%M:%S")
   
        self.Inlet1 = FluidPort(fluid='Dodecane', F=15/3.6,P=101325)
        self.Outlet1 = FluidPort(fluid='Dodecane',F=15/3.6,P=101325)
        self.Inlet2 = FluidPort(fluid='water',P=101325)
        self.Outlet2 = FluidPort(fluid='water',P=101325)

        self.T1o=None
        self.T2o=None
        self.C1=None
        self.C2=None
        self.R=None # Le rapport des débits de capacités des deux fluides est 
        self.Eff=None # efficacité


        self.P_drop=0
   
     
        self.df = pd.DataFrame()


        
    def calculate (self):
    
        self.Outlet1.P=self.Inlet1.P-self.P_drop
        self.Outlet1.F=self.Inlet1.F
        self.Outlet1.fluid=self.Inlet1.fluid
        self.Outlet2.fluid=self.Inlet2.fluid

        self.Outlet1.T=self.T1o+273.15
        self.Outlet1.calculate_properties()
        #print("_________________________________",self.Inlet1.F,self.Outlet1.h,self.Inlet1.h)
        self.Qth=self.Inlet1.F*(self.Outlet1.h-self.Inlet1.h)

        self.Outlet2.T=self.T2o+273.15
        self.Outlet2.calculate_properties()
        #print( 'self.Inlet2.F', self.Inlet2.F)
        if self.Inlet2.F is None or self.Inlet2.F==0 :
            self.Inlet2.F=abs(self.Qth/(self.Outlet2.h-self.Inlet2.h))
            self.Outlet2.F=self.Inlet2.F

        

       

        self.Inlet1.calculate_properties()
        self.Inlet2.calculate_properties()
        
        

        self.C1=self.Inlet1.F*self.Inlet1.cp
        self.C2=self.Inlet2.F*self.Inlet2.cp
        self.Cmin=min(self.C1, self.C2)
        self.Cmax=max(self.C1, self.C2)
        self.R = self.Cmin / self.Cmax


        if self.Inlet1.T>self.Inlet2.T:
            self.Eff =(self.Inlet1.T-self.Outlet1.T)/(self.Inlet1.T-self.Inlet2.T)



        #print(self.Outlet1.T-273.15,self.Inlet2.T-273.15)

        self.dTmin=min((self.Inlet1.T-self.Outlet2.T),(self.Outlet1.T-self.Inlet2.T))
        self.dTmax=max((self.Inlet1.T-self.Outlet2.T),(self.Outlet1.T-self.Inlet2.T))
        #print(self.dTmax,self.dTmin)
        #print(self.dTmax/self.dTmin)
  
        self.DTLM=(self.dTmax-self.dTmin)/math.log(self.dTmax/self.dTmin)
        #print(self.DTLM)
     
  





        self.UA=abs(self.Qth/self.DTLM)

        self.df = pd.DataFrame({'DTLM_HEX': [self.Timestamp,self.Qth], },
                      index = ['Timestamp',self.Qth])     


# from ThermodynamicCycles.Source import Source
# from ThermodynamicCycles.Connect import Fluid_connect

# SOURCE1=Source.Object()
# SOURCE1.F_m3h=15
# SOURCE1.Pi_bar=1
# SOURCE1.fluid='propane'
# SOURCE1.Ti_degC=93
# SOURCE1.calculate()

# SOURCE2=Source.Object()
# SOURCE2.Pi_bar=1
# SOURCE2.fluid='water'
# SOURCE2.Ti_degC=70
# SOURCE2.calculate()

# HEX=Object()
# HEX.T1o=70
# HEX.T2o=90
# Fluid_connect(HEX.Inlet1,SOURCE1.Outlet)
# Fluid_connect(HEX.Inlet2,SOURCE2.Outlet)
# HEX.calculate()
# print("-------------------------------",HEX.df)

# print(HEX.Outlet1.df)
# print(HEX.Outlet2.df)

# print(HEX.R)
# print(HEX.Eff)
# print(HEX.UA)


from datetime import date
import json
import os
from dateutil.parser import parse
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle

current_directory = os.path.dirname(os.path.abspath(__file__))
coeffs_atrd_path = os.path.join(current_directory, 'coefficients_gaz_ATRD.json')
coeffs_atrt_path = os.path.join(current_directory, 'coefficients_gaz_ATRT.json')
coeffs_ticgn_path = os.path.join(current_directory, 'coefficients_gaz_TICGN.json')

class input_Facture:
    def __init__(self, start, end, kWh_total=0):
        if not isinstance(start, date):
            start = parse(start).date()
        if not isinstance(end, date):
            end = parse(end).date()
        self.start = start
        self.end = end
        self.kWh_total = kWh_total

class input_Tarif:
    def __init__(self, prix_kWh=0.0, abonnement_annuel_fournisseur=0.0, distribution_cta_rate=0.0771, ticgn_rate=0.00837):
        self.prix_kWh = prix_kWh
        self.abonnement_annuel_fournisseur = abonnement_annuel_fournisseur
        self.distribution_cta_rate = distribution_cta_rate
        self.ticgn_rate = ticgn_rate

class input_Contrat:
    def __init__(self, type_tarif_acheminement='T1',CJA_MWh_j=0,CJN_MWh_j=None,modulation_MWh_j=None,CAR_MWh=0,station_meteo="PARIS-MONTSOURIS",profil="P016",reseau_transport="GRTgaz",niv_tarif_region=2, distance=None):
        self.type_tarif_acheminement = type_tarif_acheminement
        self.profil=profil
     
        self.distance = distance  # en km
        self.CJA_MWh_j=CJA_MWh_j #Capacité Journalière Annualisée
        self.CJN_MWh_j=CJN_MWh_j  # Capacité Journalière Normalisée (CJN) en MWh
        self.modulation_MWh_j = modulation_MWh_j  # Modulation journalière en MWh
        self.CAR_MWh=CAR_MWh  # Capacité Annuelle Réservée en MWh
        self.station_meteo = station_meteo  # Station météo pour le calcul des coefficients
        self.niv_tarif_region = niv_tarif_region  # Niveau tarifaire de la région (1, 2 ou 3)
        self.reseau_transport = reseau_transport  # Réseau de transport (GRTgaz ou Terega)  

def find_ticgn_rate(facture):
    with open(coeffs_ticgn_path, 'r', encoding='utf-8') as f:
        data = json.load(f)
    for coef in data["coefficients"]:
        if parse(coef["start_date"]).date() <= facture.start <= parse(coef["end_date"]).date():
            return coef["ticgn_rate"]
    raise ValueError("Aucun taux TICGN trouvé pour cette période.")      

def find_atrd_coeff(contrat, facture):
    with open(coeffs_atrd_path, 'r', encoding='utf-8') as f:
        data = json.load(f)
    for coef in data["coefficients"]:
        if (
            coef["type_tarif_acheminement"] == contrat.type_tarif_acheminement
            and parse(coef["start_date"]).date() <= facture.start <= parse(coef["end_date"]).date()
        ):
            return coef
    raise ValueError("Aucun coefficient ATRD trouvé pour cette période et ce type de tarif.")

def find_atrt_coeff(contrat, facture):
    with open(coeffs_atrt_path, 'r', encoding='utf-8') as f:
        data = json.load(f)
    for coef in data["coefficients"]:
        if (
            "start_date" in coef and "end_date" in coef
            and parse(coef["start_date"]).date() <= facture.start <= parse(coef["end_date"]).date()
        ):
            # Recherche du coefficient Zi
            station = (getattr(contrat, "station_meteo", None) or "").strip()
            profil = (getattr(contrat, "profil", None) or "").strip()
            zi_value = None
            for zi in coef.get("coefficient_zi", []):
                if zi.get("station_meteo") == station and profil in zi:
                    zi_value = zi[profil]
                    break
            if zi_value is None:
                raise ValueError(f"Pas de coefficient Zi pour la station {station} et le profil {profil}.")
            return coef, zi_value
    raise ValueError("Aucun coefficient ATRT trouvé pour cette période.")

def calcul_prix_molecule_gaz(facture, tarif):
    """
    Calcule le prix de la molécule de gaz (part fournisseur) en euros.
    :param facture: instance de input_Facture
    :param tarif: instance de input_Tarif (prix_kWh doit être renseigné)
    :return: montant en euros
    """
    return round(facture.kWh_total * tarif.prix_kWh, 2)





class ATR_calculation:
    """Calcul simultané de la part distribution (ATRD) et transport (ATRT) pour le gaz naturel."""

    def __init__(self, contrat, facture, tarif=None):
        self.contrat = contrat
        self.facture = facture
        self.tarif = tarif
        self.nb_jour = (self.facture.end - self.facture.start).days + 1

        # --- ATRD ---
        self.coeff_atrd = find_atrd_coeff(contrat, facture)
        self.euro_molecule_gaz = 0.0
        self.euro_terme_distance = 0.0
        self.euro_CTA = 0.0
        self.euro_an_CTA = 0.0
        self.euro_TICGN = 0.0
        self.euro_total_HTVA = 0.0
        self.euro_total_TTC = 0.0
        self.taxes_contributions = 0.0
        self.euro_ATRD_fixe_hors_capacite= 0.0
        self.euro_ATRD_variable = 0.0
        self.euro_ATRD_souscript_capa_CJA = 0.0
        self.euro_ATRD_fixe_total = 0.0
        self.euro_an_ATRD_souscript_capa_CJA = 0.0
        self.euro_ATRD_fixe_hors_capacite = 0.0
        self.euro_an_ATRD_fixe_total = 0.0
        self.euro_an_ATRD_variable = 0.0
        self.euro_an_ATRD_total = 0.0

        # --- ATRT ---
        self.coeff_atrt, self.zi = find_atrt_coeff(contrat, facture)
        self.coef_A_GRTgaz = self.coeff_atrt.get("coef_A_GRTgaz", 1.0)
        self.coef_A_Terega = self.coeff_atrt.get("coef_A_Terega", 1.0)
        self.coef_A = None
        self.modulation_hivernale = 0.0
        self.TCS = 0.0
        self.TCR = 0.0
        self.TCL = 0.0
        self.NTR = 0.0
        self.euro_fixe_transport = 0.0
        self.euro_variable_transport = 0.0
        self.euro_an_compensation_stockage = 0.0
        self.euro_an_ATRT_TCS = 0.0
        self.euro_an_ATRT_TCR = 0.0
        self.euro_an_ATRT_TCL = 0.0
        self.euro_an_total_ATRT = 0.0
        self.euro_ATRT_compensation_stockage = 0.0
        self.euro_ATRT_TCS = 0.0
        self.euro_ATRT_TCR = 0.0
        self.euro_ATRT_TCL = 0.0
        self.euro_total_ATRT = 0.0

        self.df = pd.DataFrame(columns=["Composante", "Montant (€)"])
        self.df_transport = pd.DataFrame(columns=["Composante", "Montant (€)"])
        self.df_distribution = pd.DataFrame(columns=["Composante", "Montant (€)"])
        self.df_taxes_contributions = pd.DataFrame(columns=["Composante", "Montant (€)"])
        self.df_molecule = pd.DataFrame(columns=["Composante", "Montant (€)"])
        self.df_annuel = pd.DataFrame(columns=["Composante", "Montant (€)"])


    def calculate(self):
        if self.contrat.reseau_transport == "GRTgaz":
            self.coef_A = self.coef_A_GRTgaz
            self.TCR = self.coeff_atrt.get("TCR_GRTgaz", 0.0)
            self.TCL = self.coeff_atrt.get("TCL_GRTgaz", 0.0)
        elif self.contrat.reseau_transport == "Terega":
            self.coef_A = self.coef_A_Terega
            self.TCR = self.coeff_atrt.get("TCR_Terega", 0.0)
            self.TCL = self.coeff_atrt.get("TCL_Terega", 0.0)
        else:
            self.coef_A = 1
        
        
        if self.contrat.CAR_MWh is not None and self.contrat.CAR_MWh > 0 and self.contrat.CJN_MWh_j is None:
            self.CJN_MWh_j = self.contrat.CAR_MWh * self.zi * self.coef_A
        else:
            self.CJN_MWh_j = self.contrat.CJN_MWh_j if self.contrat.CJN_MWh_j is not None else 0.0




        self.euro_an_ATRD_fixe_hors_capacite = self.coeff_atrd["ATRD_fixe"]
        #print("self.euro_an_ATRD_fixe_hors_capacite", self.euro_an_ATRD_fixe_hors_capacite)

        if self.nb_jour>31:
            self.euro_ATRD_fixe_hors_capacite = round(self.euro_an_ATRD_fixe_hors_capacite * self.nb_jour / 365.0, 2)
        else:
            self.euro_ATRD_fixe_hors_capacite= round(self.euro_an_ATRD_fixe_hors_capacite / 12, 2)  # Mensuel


        self.euro_an_ATRD_variable = self.coeff_atrd["prix_proportionnel_euro_kWh"] * self.facture.kWh_total

        if self.contrat.type_tarif_acheminement in ["T1", "T2", "T3"]:
            self.euro_terme_distance = 0.0
            self.euro_cta_base = self.euro_ATRD_fixe_hors_capacite
            self.euro_an_cta_base = self.euro_an_ATRD_fixe_total
            self.euro_an_ATRD_souscript_capa_CJA = 0.0
            self.euro_an_ATRD_fixe_total = self.euro_an_ATRD_fixe_hors_capacite
            self.euro_ATRD_souscript_capa_CJA = 0.0
        elif self.contrat.type_tarif_acheminement == "T4":
            #CJN_MWh_j = self.contrat.CJN_MWh_j or 0
            if self.CJN_MWh_j > 500:
                tarif_capacite = self.coeff_atrd["souscription_annuelle_capacite_euro_kWh_j_supp500"]
            else:
                tarif_capacite = self.coeff_atrd["souscription_annuelle_capacite_euro_kWh_j_inf500"]
            self.euro_an_ATRD_souscript_capa_CJA = round(self.CJN_MWh_j * 1000 * tarif_capacite, 2)
            self.euro_an_ATRD_fixe_total = self.euro_an_ATRD_fixe_hors_capacite + self.euro_an_ATRD_souscript_capa_CJA
            
            if self.nb_jour>31:
                self.euro_ATRD_souscript_capa_CJA = round(self.euro_an_ATRD_souscript_capa_CJA * self.nb_jour / 365.0, 2)
                self.euro_ATRD_fixe_total = round(self.euro_an_ATRD_fixe_total * self.nb_jour / 365.0, 2)
            else:
                self.euro_ATRD_souscript_capa_CJA=round(self.euro_an_ATRD_souscript_capa_CJA / 12, 2)  # Mensuel
                self.euro_ATRD_fixe_total=round(self.euro_an_ATRD_fixe_total / 12, 2)  # Mensuel

            self.euro_terme_distance = 0.0
            self.euro_cta_base = self.euro_ATRD_fixe_hors_capacite
            self.euro_an_cta_base = self.euro_an_ATRD_fixe_total
        elif self.contrat.type_tarif_acheminement == "TP":
            self.CJN_MWh_j = self.contrat.CJN_MWh_j or 0
            dist = self.contrat.distance or 0
            tarif_capacite = self.coeff_atrd["tarif_capacite"]
            tarif_distance = self.coeff_atrd["tarif_distance"]
            self.euro_an_ATRD_souscript_capa_CJA = round(self.CJN_MWh_j * tarif_capacite * self.nb_jour, 2)
            self.euro_terme_distance = round(dist * (tarif_distance / 365) * self.nb_jour, 2)
            self.euro_an_ATRD_fixe_total = self.euro_an_ATRD_fixe_total + self.euro_an_ATRD_souscript_capa_CJA
            self.euro_cta_base = self.euro_ATRD_fixe_hors_capacite + self.euro_terme_distance
            self.euro_an_cta_base = self.euro_an_ATRD_fixe_total + self.euro_terme_distance
        else:
            raise ValueError("Type de tarif inconnu")

        if self.tarif is not None:
            self.euro_molecule_gaz = round(self.facture.kWh_total * self.tarif.prix_kWh, 2)

        self.euro_an_ATRD_total = self.euro_an_ATRD_fixe_total + self.euro_an_ATRD_variable
        self.euro_ATRD_variable = round(self.coeff_atrd["prix_proportionnel_euro_kWh"] * self.facture.kWh_total, 2)
        self.euro_ATRD_total = round(self.euro_ATRD_fixe_total + self.euro_ATRD_variable, 2)

        self.ticgn_rate = find_ticgn_rate(self.facture)
        print("TICGN Rate:", self.ticgn_rate)
        self.euro_TICGN = round(self.facture.kWh_total * self.ticgn_rate, 2)

    
       
        

        # --- ATRT ---
 

               #Modulation Hivernale = Capacité de transport – (CAR/365)
        #print("self.modulation_MWh_j", self.contrat.modulation_MWh_j)
        if self.contrat.modulation_MWh_j is None:
            self.modulation_hivernale = self.CJN_MWh_j - (self.contrat.CAR_MWh / 365.0) if self.contrat.CAR_MWh else 0.0
            #self.modulation_hivernale = self.CJN_MWh_j - (self.contrat.CAR_MWh / 365.0) if self.contrat.CAR_MWh else 0.0
        else:
            self.modulation_hivernale=self.contrat.modulation_MWh_j

        #print("self.modulation_hivernale", self.modulation_hivernale)
        
        
        self.TCS = self.coeff_atrt["TCS"]
        self.NTR = self.contrat.niv_tarif_region
        self.coef_stockage = self.coeff_atrt.get("coef_compensation_stockage", 0)
        self.euro_an_compensation_stockage = round(self.modulation_hivernale * self.coef_stockage, 2) if self.modulation_hivernale else 0.0
        #print("self.euro_an_compensation_stockage=", self.modulation_hivernale, "*", self.coef_stockage, "=", self.euro_an_compensation_stockage)
        
        self.euro_an_ATRT_TCS = self.CJN_MWh_j * self.TCS
        self.euro_an_ATRT_TCR = self.CJN_MWh_j * (self.TCR * self.NTR)
        self.euro_an_ATRT_TCL = self.CJN_MWh_j * self.TCL
        self.euro_MWh_ATRT=self.TCS+(self.TCR * self.NTR)+self.TCL
        self.euro_MWh_mois_ATRT=self.euro_MWh_ATRT/12
        #print("self.euro_MWh_ATRT",self.euro_MWh_ATRT)
        #print("self.euro_MWh_mois_ATRT",self.euro_MWh_mois_ATRT)
        self.euro_an_total_ATRT = self.euro_an_ATRT_TCS + self.euro_an_ATRT_TCR + self.euro_an_ATRT_TCL + self.euro_an_compensation_stockage
        self.euro_an_total_ATRT_hors_compensation = self.euro_an_ATRT_TCS + self.euro_an_ATRT_TCR + self.euro_an_ATRT_TCL
        #self.euro_ATRT_compensation_stockage = self.euro_an_compensation_stockage * self.nb_jour / 365.0 if self.euro_an_compensation_stockage else 0.0
        self.euro_ATRT_compensation_stockage = round(self.euro_an_compensation_stockage /12,2) if self.euro_an_compensation_stockage else 0.0
        
        if self.nb_jour>31:
            self.euro_ATRT_TCS = self.euro_an_ATRT_TCS * self.nb_jour / 365.0 if self.euro_an_ATRT_TCS else 0.0
            self.euro_ATRT_TCR = self.euro_an_ATRT_TCR * self.nb_jour / 365.0 if self.euro_an_ATRT_TCR else 0.0
            self.euro_ATRT_TCL = self.euro_an_ATRT_TCL * self.nb_jour / 365.0 if self.euro_an_ATRT_TCL else 0.0
        
        else:
            self.euro_ATRT_TCS = self.euro_an_ATRT_TCS/12 if self.euro_an_ATRT_TCS else 0.0
            self.euro_ATRT_TCR = self.euro_an_ATRT_TCR/12 if self.euro_an_ATRT_TCR else 0.0
            self.euro_ATRT_TCL = self.euro_an_ATRT_TCL/12 if self.euro_an_ATRT_TCL else 0.0

        self.euro_total_ATRT = self.euro_ATRT_TCS + self.euro_ATRT_TCR + self.euro_ATRT_TCL + self.euro_ATRT_compensation_stockage
        self.euro_total_ATRT_hors_compensation = round(self.euro_ATRT_TCS + self.euro_ATRT_TCR + self.euro_ATRT_TCL,2)
        #Formule : CTA = [Quote-part distribution] × (20,80 % + [Coefficient] × 4,71 %).
        #    "distribution_cta_rate": 0.2080,
        #   "transport_cta_rate": 0.0471,
        #   "coefficient_proportionnalite_cta":83.21,

        self.euro_an_CTA = round(self.euro_an_ATRD_fixe_total * (self.coeff_atrd["distribution_cta_rate"]+self.coeff_atrd["coefficient_proportionnalite_cta"]*self.coeff_atrd["transport_cta_rate"]), 2) #corriger
        
        if self.nb_jour>31:
            self.euro_CTA = self.euro_an_CTA * self.nb_jour / 365.
        else:
            self.euro_CTA = round(self.euro_an_CTA / 12, 3)  # Mensuel

        self.taxes_contributions = round(self.euro_CTA + self.euro_TICGN, 2)

        self.euro_acheminement_transport_distribution = self.euro_total_ATRT_hors_compensation + self.euro_ATRT_compensation_stockage+ self.euro_ATRD_fixe_hors_capacite+self.euro_ATRD_souscript_capa_CJA+self.euro_ATRD_variable
     
        self.euro_total_HTVA = round(
            self.euro_acheminement_transport_distribution+
            self.euro_molecule_gaz+
            self.taxes_contributions, 2
        )

        self.euro_TVA_5_5 = round((self.euro_total_ATRT_hors_compensation +
            self.euro_ATRT_compensation_stockage+
                                   
            self.euro_ATRD_fixe_hors_capacite 
            +self.euro_ATRD_souscript_capa_CJA
            + self.euro_CTA) * 0.055, 2)

        self.euro_TVA_20 = round((self.euro_ATRD_variable 
                                  + self.euro_molecule_gaz 
                                  + self.euro_TICGN
                                  ) * 0.20, 2)
        
        self.euro_TVA = self.euro_TVA_5_5 + self.euro_TVA_20
        self.euro_total_TTC = round(self.euro_total_HTVA + self.euro_TVA, 2)

        self.cout_variable_ATRD_euro_par_MWh=round(self.euro_ATRD_variable / (self.facture.kWh_total / 1000),2) if self.facture.kWh_total else 0
        self.cout_TICGN_euro_par_MWh=round(self.euro_TICGN / (self.facture.kWh_total / 1000),2) if self.facture.kWh_total else 0
        self.cout_variable_ATRD_plus_TICGN_euro_par_MWh=round(self.cout_variable_ATRD_euro_par_MWh + self.cout_TICGN_euro_par_MWh,2)
        self.cout_molecule_gaz_euro_par_MWh = round(self.euro_molecule_gaz / (self.facture.kWh_total / 1000),2) if self.facture.kWh_total else 0
        self.cout_euro_HTVA_par_MWh =round(self.euro_total_HTVA / (self.facture.kWh_total / 1000),2) if self.facture.kWh_total else 0
        # print("cout_variable_ATRD_euro_par_MWh", self.cout_variable_ATRD_euro_par_MWh)
        # print("cout_TICGN_euro_par_MWh", self.cout_TICGN_euro_par_MWh)
        # print("cout_molecule_gaz_euro_par_MWh", self.cout_molecule_gaz_euro_par_MWh)
        # print("cout_variable_ATRD_plus_TICGN_euro_par_MWh", self.cout_variable_ATRD_plus_TICGN_euro_par_MWh)
        # print("cout_euro_HTVA_par_MWh", self.cout_euro_HTVA_par_MWh)
         
        

        self.resume_all()

    def resume_all(self):
        import pandas as pd
        self.df_euro_MWh=pd.DataFrame([
            ("Cout variable ATRD (€ par MWh) : ", self.cout_variable_ATRD_euro_par_MWh),
            ("Cout TICGN (€ par MWh) : ", self.cout_TICGN_euro_par_MWh),
            ("Cout variable ATRD + TICGN (€ par MWh) : ", self.cout_variable_ATRD_plus_TICGN_euro_par_MWh),
            ("Cout de la molécule de gaz (€ par MWh) : ", self.cout_molecule_gaz_euro_par_MWh),
            ("Cout HTVA (€ par MWh) : ", self.cout_euro_HTVA_par_MWh),
        ], columns=["Composante", "Montant (€)"])

        self.df=pd.DataFrame([
            ("self.euro_acheminement_transport_distribution (€/mois) : ", self.euro_acheminement_transport_distribution),
            
            ("-ATRD mensuel total (€/mois) : ", self.euro_ATRD_total),
            ("-Total ATRT (€/mois)", self.euro_total_ATRT),

            ("Prix de la molécule de gaz (€) : ", self.euro_molecule_gaz),
            ("Taxes et contributions (€) : ", self.taxes_contributions),

            ("Total HTVA (€) : ", self.euro_total_HTVA),
            ("TVA 5,5% (€) : ", self.euro_TVA_5_5),
            ("TVA 20% (€) : ", self.euro_TVA_20),
            ("Total TVA (€) : ", self.euro_TVA),
            ("Total TTC (€) : ", self.euro_total_TTC),
            
        ], columns=["Composante", "Montant (€)"])

        #self.df_transport = pd.DataFrame(columns=["Composante", "Montant (€)"])
        #mois
        self.df_transport=pd.DataFrame([
                ("Total ATRT (€/mois)", self.euro_total_ATRT),
                ("-ATRT - Réseau principal (€/mois)", self.euro_ATRT_TCS),
                ("-ATRT - Réseau régional (€/mois)", self.euro_ATRT_TCR),
                ("-ATRT - Souscription de capacité journalière de livraison (€/mois)", self.euro_ATRT_TCL),
                ("ATRT_hors_compensation (€/mois)", self.euro_total_ATRT_hors_compensation),
                ("-ATRT - Compensation stockage (€/mois)", self.euro_ATRT_compensation_stockage),
                
             
               
            ], columns=["Composantes ATRT", "Montant (€)"])


        #self.df_distribution = pd.DataFrame(columns=["Composante", "Montant (€)"])
        #mois
        self.df_distribution=pd.DataFrame([
                ("ATRD fixe total (€/mois)", self.euro_ATRD_fixe_total),
                ("- ATRD fixe (€/mois)-hors souscrip cap.", self.euro_ATRD_fixe_hors_capacite),
                ("- ATRD Souscription de capacité  (€/mois)", self.euro_ATRD_souscript_capa_CJA),
                ("- ATRD variable (€/mois)", self.euro_ATRD_variable),
            ], columns=["Composante", "Montant (€)"])
        

        #self.df_taxes_contributions = pd.DataFrame(columns=["Composante", "Montant (€)"])
        #mois
        self.df_taxes_contributions=pd.DataFrame([
                ("Taxes et contributions (€)", self.taxes_contributions),
                ("- CTA (€)", self.euro_CTA),
                ("- TICGN (€)", self.euro_TICGN),
            ], columns=["Composante", "Montant (€)"])
        
        #self.df_molecule = pd.DataFrame(columns=["Composante", "Montant (€)"])
        #mois
        self.df_molecule=pd.DataFrame([
                ("Prix de la molécule de gaz (€)", self.euro_molecule_gaz),
            ], columns=["Composante", "Montant (€)"])
        #self.df_annuel = pd.DataFrame(columns=["Composante", "Montant (€)"])
        #annuel
        self.df_annuel=pd.DataFrame([
             ("self.euro_an_total_ATRT (€/an)", self.euro_an_total_ATRT),
            ("ATRT - Compensation stockage (€/an)", self.euro_an_compensation_stockage),
            ("total_ATRT_hors_compensation (€/an)", self.euro_an_total_ATRT_hors_compensation),
            ('ATRD_fixe_hors_capacite hors capacite" (€/an)', self.euro_an_ATRD_fixe_hors_capacite),

                ("ATRD total (€/an)", self.euro_an_ATRD_total),
                ("ARTD stockage annuel (€/an)", self.euro_an_compensation_stockage),
                ("ATRD fixe (€/an)", self.euro_an_ATRD_fixe_total),
                ("ATRD variable (€/an)", self.euro_an_ATRD_variable),
                ("CTA annuel (€)", self.euro_an_CTA),
                ("TICGN annuel (€)", self.euro_TICGN),
                ("Total HTVA (€)", self.euro_total_HTVA),
                ("TVA 5,5% (€)", self.euro_TVA_5_5),
                ("TVA 20% (€)", self.euro_TVA_20),
                ("Total TVA (€)", self.euro_TVA),
                ("Total TTC (€)", self.euro_total_TTC),
            ], columns=["Composantes annuelles", "Montant (€)"])
      
     
    def plot(self, title="Facture Gaz : les composantes principales", figsize=(8, 6)):
        """Affiche un donut plot de la répartition ATRD/ATRT/Taxes."""
        labels = ['Acheminement (Distribution et Transport)', 'Consommation de gaz (Fournisseur)','Taxes et Contributions']
        values = [self.euro_ATRD_total+self.euro_total_ATRT, self.euro_molecule_gaz,self.taxes_contributions ]
        colors = ['#4CAF50', '#FFC107', '#FF5722']

        total = sum(values)
        fig, ax = plt.subplots(figsize=figsize)
        wedges, texts, autotexts = ax.pie(
            values,
            labels=None,
            autopct='%1.1f%%',
            startangle=90,
            colors=colors,
            wedgeprops=dict(width=0.4, edgecolor='w'),
            pctdistance=0.8,
            textprops={'fontsize': 12}
        )
        ax.axis('equal')
        plt.title(title, fontsize=12, pad=16)
        ax.text(0, 0, f"{total:,.2f} €", ha='center', va='center', fontsize=14, fontweight='bold')

        # Légende
        y_start = -0.05
        y_step = 0.07
        x_rect = -0.12
        x_text = -0.05
        for i, (label, val, color) in enumerate(zip(labels, values, colors)):
            y = y_start - i * y_step
            rect = Rectangle((x_rect, y - 0.02), 0.04, 0.04, facecolor=color, transform=ax.transAxes, clip_on=False)
            ax.add_patch(rect)
            ax.text(x_text, y, f"{label} : {val:,.2f} €",
                    transform=ax.transAxes,
                    ha='left', va='center',
                    fontsize=11, weight='bold')
        plt.tight_layout()
        plt.show()


    def plot_detail(self, title="Détail des Composantes de la Facture Gaz (ATRD/ATRT/Taxes et Contributions)", figsize=(16, 6)):
        # Vérification des données nécessaires
        necessary_attrs = [
            'euro_ATRD_fixe_hors_capacite', 'euro_ATRD_souscript_capa_CJA', 'euro_ATRD_variable',
            'euro_ATRT_TCS', 'euro_ATRT_TCR', 'euro_ATRT_TCL', 'euro_ATRT_compensation_stockage',
            'euro_CTA', 'euro_TICGN'
        ]
        for attr in necessary_attrs:
            if getattr(self, attr, None) is None:
                print(f"Attribut {attr} non défini.")
                return

        # === Distribution (ATRD) ===
        atrd_labels = ['Fixe (hors capacité)', 'Souscription capacité', 'Variable']
        atrd_values = [
            self.euro_ATRD_fixe_hors_capacite,
            self.euro_ATRD_souscript_capa_CJA,
            self.euro_ATRD_variable
        ]

        # === Transport (ATRT) ===
        atrt_labels = ['Réseau principal TCS', 'Régional TCR', 'Capacité livraison TCL', 'Compensation stockage TS']
        atrt_values = [
            self.euro_ATRT_TCS,
            self.euro_ATRT_TCR,
            self.euro_ATRT_TCL,
            self.euro_ATRT_compensation_stockage
        ]

        # === Taxes et Contributions ===
        taxes_labels = ['CTA', 'TICGN']
        taxes_values = [
            self.euro_CTA,
            self.euro_TICGN
        ]

        fig, axs = plt.subplots(1, 3, figsize=figsize)

        def plot_waterfall(ax, labels, values, title):
            cum_values = [0]
            for v in values:
                cum_values.append(cum_values[-1] + v)

            colors = ['green' if v >= 0 else 'red' for v in values]
            for i, (label, val) in enumerate(zip(labels, values)):
                ax.bar(i, val, bottom=cum_values[i], color=colors[i], edgecolor='black')
                y_text = cum_values[i] + val / 2
                ax.text(i, y_text, f"{val:.2f} €", ha='center', va='center', fontsize=9,
                        color='black' if abs(val) > 20 else 'black')

            total = sum(values)
            ax.set_xticks(range(len(labels)))
            ax.set_xticklabels(labels, rotation=45, ha='right')
            ax.axhline(0, color='black', linewidth=0.8)
            ax.set_title(f"{title}\nTotal : {total:.2f} €", fontsize=12)
            ax.grid(axis='y', linestyle='--', alpha=0.5)

        # Tracer les 3 cascades
        plot_waterfall(axs[0], atrd_labels, atrd_values, "Distribution (ATRD)")
        plot_waterfall(axs[1], atrt_labels, atrt_values, "Transport (ATRT)")
        plot_waterfall(axs[2], taxes_labels, taxes_values, "Taxes et Contributions")

        plt.suptitle(title, fontsize=16, y=0.97)
        plt.tight_layout(rect=[0, 0, 1, 0.93])
        plt.show()

    def plot_euro_MWh(self, title="Décomposition du coût en €/MWh (HTVA)", figsize=(8, 6)):
        labels = [
            "Variable ATRD (€/MWh)",
            "TICGN (€/MWh)",
            "Molécule gaz (€/MWh)",
            "Total HTVA (€/MWh)"
        ]
        # Accumulation pour les 3 premières composantes
        values = [
            self.cout_variable_ATRD_euro_par_MWh,
            self.cout_TICGN_euro_par_MWh,
            self.cout_molecule_gaz_euro_par_MWh
        ]
        cum_values = [0]
        for v in values:
            cum_values.append(cum_values[-1] + v)
        # La 4e barre (total HTVA) n'est pas accumulée
        total_htva = self.cout_euro_HTVA_par_MWh

        colors = ['#4CAF50', '#FFC107', '#2196F3', '#9E9E9E']
        fig, ax = plt.subplots(figsize=figsize)

        # Barres en cascade (accumulation)
        for i, (label, val, color) in enumerate(zip(labels[:3], values, colors)):
            ax.bar(i, val, bottom=cum_values[i], color=color, edgecolor='black')
            y_text = cum_values[i] + val / 2
            ax.text(i, y_text, f"{val:.2f} €/MWh", ha='center', va='center', fontsize=10, color='black')

        # Barre du total HTVA (non accumulée)
        ax.bar(3, total_htva, color=colors[3], edgecolor='black')
        ax.text(3, total_htva / 2, f"{total_htva:.2f} €/MWh", ha='center', va='center', fontsize=10, color='black')

        ax.set_xticks(range(len(labels)))
        ax.set_xticklabels(labels, rotation=30, ha='right')
        ax.axhline(0, color='black', linewidth=0.8)
        ax.set_title(f"{title}\nTotal HTVA : {self.cout_euro_HTVA_par_MWh:.2f} €/MWh", fontsize=12)
        ax.grid(axis='y', linestyle='--', alpha=0.5)
        plt.tight_layout()
        plt.show()


from requests.packages import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

# from bs4 import BeautifulSoup
import pandas as pd
# from datetime import datetime

from MeteoCiel.MeteoCiel_dayScraping import MeteoCiel_dayScraping
from MeteoCiel.DJU_costic import DJU_costic

#############"aller chercher toutes les données historique########################
def MeteoCiel_histoScraping(code2,date_debut,date_fin,base_chauffage=18, base_refroidissement=23):
    from tqdm import tqdm
    from datetime import datetime

    df_histo=pd.DataFrame([])
    date_reference=datetime(1970,1,1,1,0,0,0)
    Timestamp_debut=round((date_debut-date_reference).total_seconds())
    Timestamp_fin=round((date_fin-date_reference).total_seconds())

    #for annee2 in range(date_debut.year, date_fin.year+1):
    for current_Timestamp in tqdm(range(Timestamp_debut, Timestamp_fin,3600*24),desc='Evolution du temps de scraping'):
        current_date=datetime.fromtimestamp(current_Timestamp)
        try:
            df=MeteoCiel_dayScraping(code2,current_date.year,current_date.month,current_date.day)
            #print("df=====MeteoCiel_dayScraping================",df)
            
            df_histo=df_histo.append(df)       
        except:
            print("erreur dans le scraping de la journée :"+str(current_date))
            df=MeteoCiel_dayScraping(code2,current_date.year,current_date.month,current_date.day)
            df_histo=df_histo._append(df) 
                
        #print("df_histo=====",df_histo)
    #df_histo.to_excel("fichier_meteociel_scap.xlsx", index=False) 
    print("fin du scrapping...........")


    date='Timestamp'
    # Convert the column to numeric with errors='coerce'
    #print(df_histo)
    try:
        df_histo['Température'] = pd.to_numeric(df_histo['Température'], errors='coerce')
    except:
        print("les données de température n'ont pas été numérisées")
    print(df_histo.columns)

    try:
        df_histo = df_histo.dropna(subset=['Température'])
    except KeyError:
        print("Erreur: La colonne 'Température' n'existe pas dans df_histo.")

    # Identify the rows containing NaN values
    #problematic_rows = df[df['Température'].isna()]
    # Print the problematic rows
    #print(problematic_rows)

    #créer un index pour l'agregation
    #df_histo['date_only'] = df_histo[date].dt.strftime('%Y-%m-%d')
    df_histo['date_only']=df_histo.index
    df_histo['month_only']=df_histo.index
    df_histo['year_only']=df_histo.index

    df_histo['date_only'] =df_histo['date_only'].apply(lambda x: x.replace(hour=0, minute=0, second=0))
    #df_histo['month_only'] = df_histo[date].dt.strftime('%Y-%m')
    df_histo['month_only'] =df_histo['date_only'].apply(lambda x: x.replace(day=1,hour=0, minute=0, second=0))
    #df_histo['year_only'] = df_histo[date].dt.strftime('%Y')
    df_histo['year_only'] =df_histo['date_only'].apply(lambda x: x.replace(month=1,day=1,hour=0, minute=0, second=0))

    #print("df_histo.head()=====",df_histo.head())

    #calcul DJU 
    df_day=pd.DataFrame([])
    df_month=pd.DataFrame([])
    df_year=pd.DataFrame([])
    #calcul de la table jour
    if not df_histo.empty:
        df_day = df_histo.groupby('date_only').agg({'Température': ['mean','min', 'max']}) 
        df_day['DJU_Chauffage'], df_day['DJU_Rafraichissement'] = zip(*df_day.apply(
            lambda row: DJU_costic(row[('Température', 'min')], row[('Température', 'max')],base_chauffage=base_chauffage, base_refroidissement=base_refroidissement),
            axis=1
        ))

        df_day['month_only']=df_histo.groupby('date_only').agg({'month_only': ['first']})
        df_day['year_only']=df_histo.groupby('date_only').agg({'year_only': ['first']})
    
        #print(df_day.columns)

        #calcul de la table mois
        #df_month= df_day.groupby('month_only').agg({('DJU_Chauffage',''): ['sum']})
        df_month = df_day.groupby('month_only').agg({('DJU_Chauffage',''): ['sum'],('DJU_Rafraichissement',''): ['sum']})
        df_month['Température']=df_histo.groupby('month_only').agg({'Température': ['mean']})

        #print(df_month)

        #calcul de la table année
        #df_year= df_day.groupby('year_only').agg({('DJU_Chauffage',''): ['sum']})
        df_year= df_day.groupby('year_only').agg({('DJU_Chauffage',''): ['sum'],('DJU_Rafraichissement',''): ['sum']})
        df_year['Température']=df_histo.groupby('year_only').agg({'Température': ['mean']})

        #print(df_year)

        df_day.columns = [ 'Température_moyenne', 'Température_min', 'Température_max',
    'DJU_Chauffage', 'DJU_Rafraichissement', 'Month_only', 'Year_only']
        df_month.columns = ['DJU_Chauffage', 'DJU_Rafraichissement', 'Température']


   

 
    return df_histo,df_day, df_month, df_year
    








from .Two_flow_pinch_analysis import Two_flow_pinch_analysis  # Import the function
import pandas as pd

def check_additional_heat_exchanges(remain_streams):
    possible_exchanges = []
    tested_combinations = set()  # Set to track tested combinations
    # Debug: Print the actual remaining streams being passed
    # print("\n1. Remaining streams before testing:=============check_additional_heat_exchanges=====================================\n")
    # print(remain_streams)

    # Créer une copie de remain_streams pour éviter les modifications en place
    updated_remain_streams = remain_streams.copy()

    # Boucle sur toutes les combinaisons possibles de flux chauds et froids
    while True:
        # Filtrer les flux chauds et froids restants
        hot_streams = updated_remain_streams[updated_remain_streams['StreamType'] == 'HS']
        cold_streams = updated_remain_streams[updated_remain_streams['StreamType'] == 'CS']

        # Si aucun flux chaud ou froid n'est disponible, arrêter la boucle
        if hot_streams.empty or cold_streams.empty:
            break

        # Variable pour suivre si un échange a été trouvé dans cette itération
        exchange_found = False

        for _, hot_stream in hot_streams.iterrows():
            for _, cold_stream in cold_streams.iterrows():

                                # Identifier la combinaison actuelle
                combination = (hot_stream['id'], cold_stream['id'])

                # Vérifier si la combinaison a déjà été testée
                if combination in tested_combinations:
                    continue  # Passer à la prochaine combinaison

                # Ajouter la combinaison au set des combinaisons testées
                tested_combinations.add(combination)
                
                # Convertir les Series en DataFrames à une seule ligne
                hot_stream_df = hot_stream.to_frame().T
                cold_stream_df = cold_stream.to_frame().T
                #print(f"Testing exchange between HS_id={hot_stream['id']} and CS_id={cold_stream['id']}")
                # Appeler Two_flow_pinch_analysis avec les DataFrames
                try:
                    results = Two_flow_pinch_analysis(hot_stream_df, cold_stream_df)
                    if results["exchanger"]["HeatExchanged"] > 0.0:
                        possible_exchanges.append(results["exchanger"])
                        exchange_found = True

                        # === Mise à jour de updated_remain_streams ===
                        # Étape 1 : suppression des lignes à remplacer
                        updated_remain_streams.drop(
                            updated_remain_streams[updated_remain_streams['id'].isin(results["remaining_fluxes"]['id'])].index,
                            inplace=True
                        )

                        # Étape 2 : concaténation des nouvelles lignes
                        updated_remain_streams = pd.concat(
                            [updated_remain_streams, results["remaining_fluxes"]],
                            ignore_index=True
                        )
                        break  # Sortir de la boucle interne après un échange réussi
                except Exception as e:
                    print(f"Error during analysis for HS_id={hot_stream['id']} and CS_id={cold_stream['id']}: {e}")
            if exchange_found:
                break  # Sortir de la boucle externe après un échange réussi

        # Si aucun échange n'a été trouvé, arrêter la boucle
        if not exchange_found:
            break

    # Filtrer les échanges avec HeatExchanged > 0.0
    possible_exchanges = [exchange for exchange in possible_exchanges if exchange["HeatExchanged"] > 0.0]

    # Debug: Print all possible exchanges found
    # print("\n2. Possible additional exchanges found:=============check_additional_heat_exchanges=====================================\n")
    # print(possible_exchanges)
    # print("\n")
        
    return possible_exchanges, updated_remain_streams



# -*- coding: utf-8 -*-
"""
A module containing NodeEditor's class for representing Socket and Socket Position Constants.
"""
from collections import OrderedDict
from NodeEditor.nodeeditor.node_serializable import Serializable
from NodeEditor.nodeeditor.node_graphics_socket import QDMGraphicsSocket


LEFT_TOP = 1        #:
LEFT_CENTER =2      #:
LEFT_BOTTOM = 3     #:
RIGHT_TOP = 4       #:
RIGHT_CENTER = 5    #:
RIGHT_BOTTOM = 6    #:


DEBUG = False
DEBUG_REMOVE_WARNINGS = False


class Socket(Serializable):
    Socket_GR_Class = QDMGraphicsSocket

    """Class representing Socket."""

    def __init__(self, node:'Node', index:int=0, position:int=LEFT_TOP, socket_type:int=1, multi_edges:bool=True, count_on_this_node_side:int=1, is_input:bool=False):
        """
        :param node: reference to the :class:`~nodeeditor.node_node.Node` containing this `Socket`
        :type node: :class:`~nodeeditor.node_node.Node`
        :param index: Current index of this socket in the position
        :type index: ``int``
        :param position: Socket position. See :ref:`socket-position-constants`
        :param socket_type: Constant defining type(color) of this socket
        :param multi_edges: Can this socket have multiple `Edges` connected?
        :type multi_edges: ``bool``
        :param count_on_this_node_side: number of total sockets on this position
        :type count_on_this_node_side: ``int``
        :param is_input: Is this an input `Socket`?
        :type is_input: ``bool``

        :Instance Attributes:

            - **node** - reference to the :class:`~nodeeditor.node_node.Node` containing this `Socket`
            - **edges** - list of `Edges` connected to this `Socket`
            - **grSocket** - reference to the :class:`~nodeeditor.node_graphics_socket.QDMGraphicsSocket`
            - **position** - Socket position. See :ref:`socket-position-constants`
            - **index** - Current index of this socket in the position
            - **socket_type** - Constant defining type(color) of this socket
            - **count_on_this_node_side** - number of sockets on this position
            - **is_multi_edges** - ``True`` if `Socket` can contain multiple `Edges`
            - **is_input** - ``True`` if this socket serves for Input
            - **is_output** - ``True`` if this socket serves for Output
        """
        super().__init__()

        self.node = node
        self.position = position
        self.index = index
        self.socket_type = socket_type
        self.count_on_this_node_side = count_on_this_node_side
        self.is_multi_edges = multi_edges
        self.is_input = is_input
        self.is_output = not self.is_input


        if DEBUG: print("Socket -- creating with", self.index, self.position, "for nodeeditor", self.node)


        self.grSocket = self.__class__.Socket_GR_Class(self)

        self.setSocketPosition()

        self.edges = []

    def __str__(self):
        return "<Socket #%d %s %s..%s>" % (
            self.index, "ME" if self.is_multi_edges else "SE", hex(id(self))[2:5], hex(id(self))[-3:]
        )

    def delete(self):
        """Delete this `Socket` from graphics scene for sure"""
        self.grSocket.setParentItem(None)
        self.node.scene.grScene.removeItem(self.grSocket)
        del self.grSocket

    def changeSocketType(self, new_socket_type:int) -> bool:
        """
        Change the Socket Type

        :param new_socket_type: new socket type
        :type new_socket_type: ``int``
        :return: Returns ``True`` if the socket type was actually changed
        :rtype: ``bool``
        """
        if self.socket_type != new_socket_type:
            self.socket_type = new_socket_type
            self.grSocket.changeSocketType()
            return True
        return False

    def setSocketPosition(self):
        """Helper function to set `Graphics Socket` position. Exact socket position is calculated
        inside :class:`~nodeeditor.node_node.Node`."""
        self.grSocket.setPos(*self.node.getSocketPosition(self.index, self.position, self.count_on_this_node_side))

    def getSocketPosition(self):
        """
        :return: Returns this `Socket` position according the implementation stored in
            :class:`~nodeeditor.node_node.Node`
        :rtype: ``x, y`` position
        """
        if DEBUG: print("  GSP: ", self.index, self.position, "nodeeditor:", self.node)
        res = self.node.getSocketPosition(self.index, self.position, self.count_on_this_node_side)
        if DEBUG: print("  res", res)
        return res


    def hasAnyEdge(self) -> bool:
        """
        Returns ``True`` if any :class:`~nodeeditor.node_edge.Edge` is connectected to this socket

        :return: ``True`` if any :class:`~nodeeditor.node_edge.Edge` is connected to this socket
        :rtype: ``bool``
        """
        return len(self.edges) > 0

    def isConnected(self, edge:'Edge') -> bool:
        """
        Returns ``True`` if :class:`~nodeeditor.node_edge.Edge` is connected to this `Socket`

        :param edge: :class:`~nodeeditor.node_edge.Edge` to check if it is connected to this `Socket`
        :type edge: :class:`~nodeeditor.node_edge.Edge`
        :return: ``True`` if `Edge` is connected to this socket
        :rtype: ``bool``
        """
        return edge in self.edges

    def addEdge(self, edge:'Edge'):
        """
        Append an Edge to the list of connected Edges

        :param edge: :class:`~nodeeditor.node_edge.Edge` to connect to this `Socket`
        :type edge: :class:`~nodeeditor.node_edge.Edge`
        """
        self.edges.append(edge)

    def removeEdge(self, edge:'Edge'):
        """
        Disconnect passed :class:`~nodeeditor.node_edge.Edge` from this `Socket`
        :param edge: :class:`~nodeeditor.node_edge.Edge` to disconnect
        :type edge: :class:`~nodeeditor.node_edge.Edge`
        """
        if edge in self.edges: self.edges.remove(edge)
        else:
            if DEBUG_REMOVE_WARNINGS:
                print("!W:", "Socket::removeEdge", "wanna remove edge", edge,
                      "from self.edges but it's not in the list!")

    def removeAllEdges(self, silent=False):
        """Disconnect all `Edges` from this `Socket`"""
        while self.edges:
            edge = self.edges.pop(0)
            if silent:
                edge.remove(silent_for_socket=self)
            else:
                edge.remove()       # just remove all with notifications

    def determineMultiEdges(self, data:dict) -> bool:
        """
        Deserialization helper function. In our tutorials we create new version of graph data format.
        This function is here to help solve the issue of opening older files in the newer format.
        If the 'multi_edges' param is missing in the dictionary, we determine if this `Socket`
        should support multiple `Edges`.

        :param data: `Socket` data in ``dict`` format for deserialization
        :type data: ``dict``
        :return: ``True`` if this `Socket` should support multi_edges
        """
        if 'multi_edges' in data:
            return data['multi_edges']
        else:
            # probably older version of file, make RIGHT socket multiedged by default
            return data['position'] in (RIGHT_BOTTOM, RIGHT_TOP)

    def serialize(self) -> OrderedDict:
        return OrderedDict([
            ('id', self.id),
            ('index', self.index),
            ('multi_edges', self.is_multi_edges),
            ('position', self.position),
            ('socket_type', self.socket_type),
        ])

    def deserialize(self, data:dict, hashmap:dict={}, restore_id:bool=True) -> bool:
        if restore_id: self.id = data['id']
        self.is_multi_edges = self.determineMultiEdges(data)
        self.changeSocketType(data['socket_type'])
        hashmap[data['id']] = self
        return True


# -*- coding: utf-8 -*-
"""
A module containing Graphics representation of :class:`~nodeeditor.node_node.Node`
"""
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *


class QDMGraphicsNode(QGraphicsItem):
    """Class describing Graphics representation of :class:`~nodeeditor.node_node.Node`"""
    def __init__(self, node:'Node', parent:QWidget=None):
        """
        :param node: reference to :class:`~nodeeditor.node_node.Node`
        :type node: :class:`~nodeeditor.node_node.Node`
        :param parent: parent widget
        :type parent: QWidget

        :Instance Attributes:

            - **node** - reference to :class:`~nodeeditor.node_node.Node`
        """
        super().__init__(parent)
        self.node = node

        # init our flags
        self.hovered = False
        self._was_moved = False
        self._last_selected_state = False

        self.initSizes()
        self.initAssets()
        self.initUI()

    @property
    def content(self):
        """Reference to `Node Content`"""
        return self.node.content if self.node else None

    @property
    def title(self):
        """title of this `Node`

        :getter: current Graphics Node title
        :setter: stores and make visible the new title
        :type: str
        """
        return self._title

    @title.setter
    def title(self, value):
        self._title = value
        self.title_item.setPlainText(self._title)

    def initUI(self):
        """Set up this ``QGraphicsItem``"""
        self.setFlag(QGraphicsItem.ItemIsSelectable)
        self.setFlag(QGraphicsItem.ItemIsMovable)
        self.setAcceptHoverEvents(True)

        # init title
        self.initTitle()
        self.title = self.node.title

        self.initContent()

    def initSizes(self):
        """Set up internal attributes like `width`, `height`, etc."""
        self.width = 180
        self.height = 140
        self.edge_roundness = 10.0
        self.edge_padding = 10.0
        self.title_height = 24.0
        self.title_horizontal_padding = 4.0
        self.title_vertical_padding = 4.0

    def initAssets(self):
        """Initialize ``QObjects`` like ``QColor``, ``QPen`` and ``QBrush``"""
        self._title_color = Qt.white
        self._title_font = QFont("Ubuntu", 10)

        self._color = QColor("#7F000000")
        self._color_selected = QColor("#FFFFA637")
        self._color_hovered = QColor("#FF37A6FF")

        self._pen_default = QPen(self._color)
        self._pen_default.setWidthF(2.0)
        self._pen_selected = QPen(self._color_selected)
        self._pen_selected.setWidthF(2.0)
        self._pen_hovered = QPen(self._color_hovered)
        self._pen_hovered.setWidthF(3.0)

        self._brush_title = QBrush(QColor("#FF313131"))
        self._brush_background = QBrush(QColor("#E3212121"))

    def onSelected(self):
        """Our event handling when the node was selected"""
        self.node.scene.grScene.itemSelected.emit()

    def doSelect(self, new_state=True):
        """Safe version of selecting the `Graphics Node`. Takes care about the selection state flag used internally

        :param new_state: ``True`` to select, ``False`` to deselect
        :type new_state: ``bool``
        """
        self.setSelected(new_state)
        self._last_selected_state = new_state
        if new_state: self.onSelected()

    def mouseMoveEvent(self, event):
        """Overriden event to detect that we moved with this `Node`"""
        super().mouseMoveEvent(event)

        # optimize me! just update the selected nodes
        for node in self.scene().scene.nodes:
            if node.grNode.isSelected():
                node.updateConnectedEdges()
        self._was_moved = True

    def mouseReleaseEvent(self, event):
        """Overriden event to handle when we moved, selected or deselected this `Node`"""
        super().mouseReleaseEvent(event)

        # handle when grNode moved
        if self._was_moved:
            self._was_moved = False
            self.node.scene.history.storeHistory("Node moved", setModified=True)

            self.node.scene.resetLastSelectedStates()
            self.doSelect()     # also trigger itemSelected when node was moved

            # we need to store the last selected state, because moving does also select the nodes
            self.node.scene._last_selected_items = self.node.scene.getSelectedItems()

            # now we want to skip storing selection
            return

        # handle when grNode was clicked on
        if self._last_selected_state != self.isSelected() or self.node.scene._last_selected_items != self.node.scene.getSelectedItems():
            self.node.scene.resetLastSelectedStates()
            self._last_selected_state = self.isSelected()
            self.onSelected()

    def mouseDoubleClickEvent(self, event):
        """Overriden event for doubleclick. Resend to `Node::onDoubleClicked`"""
        self.node.onDoubleClicked(event)

    def hoverEnterEvent(self, event: 'QGraphicsSceneHoverEvent') -> None:
        """Handle hover effect"""
        self.hovered = True
        self.update()

    def hoverLeaveEvent(self, event: 'QGraphicsSceneHoverEvent') -> None:
        """Handle hover effect"""
        self.hovered = False
        self.update()


    def boundingRect(self) -> QRectF:
        """Defining Qt' bounding rectangle"""
        return QRectF(
            0,
            0,
            self.width,
            self.height
        ).normalized()


    def initTitle(self):
        """Set up the title Graphics representation: font, color, position, etc."""
        self.title_item = QGraphicsTextItem(self)
        self.title_item.node = self.node
        self.title_item.setDefaultTextColor(self._title_color)
        self.title_item.setFont(self._title_font)
        self.title_item.setPos(self.title_horizontal_padding, 0)
        self.title_item.setTextWidth(
            self.width
            - 2 * self.title_horizontal_padding
        )

    def initContent(self):
        """Set up the `grContent` - ``QGraphicsProxyWidget`` to have a container for `Graphics Content`"""
        if self.content is not None:
            # self.content.setGeometry(self.edge_padding, self.title_height + self.edge_padding,
            #                      self.width - 2 * self.edge_padding, self.height - 2 * self.edge_padding - self.title_height)

            self.content.setGeometry(int(self.edge_padding), int(self.title_height + self.edge_padding),
                          int(self.width - 2 * self.edge_padding), int(self.height - 2 * self.edge_padding - self.title_height))


        # get the QGraphicsProxyWidget when inserted into the grScene
        self.grContent = self.node.scene.grScene.addWidget(self.content)
        self.grContent.setParentItem(self)


    def paint(self, painter, QStyleOptionGraphicsItem, widget=None):
        """Painting the rounded rectanglar `Node`"""
        # title
        path_title = QPainterPath()
        path_title.setFillRule(Qt.WindingFill)
        path_title.addRoundedRect(0, 0, self.width, self.title_height, self.edge_roundness, self.edge_roundness)
        path_title.addRect(0, self.title_height - self.edge_roundness, self.edge_roundness, self.edge_roundness)
        path_title.addRect(self.width - self.edge_roundness, self.title_height - self.edge_roundness, self.edge_roundness, self.edge_roundness)
        painter.setPen(Qt.NoPen)
        painter.setBrush(self._brush_title)
        painter.drawPath(path_title.simplified())


        # content
        path_content = QPainterPath()
        path_content.setFillRule(Qt.WindingFill)
        path_content.addRoundedRect(0, self.title_height, self.width, self.height - self.title_height, self.edge_roundness, self.edge_roundness)
        path_content.addRect(0, self.title_height, self.edge_roundness, self.edge_roundness)
        path_content.addRect(self.width - self.edge_roundness, self.title_height, self.edge_roundness, self.edge_roundness)
        painter.setPen(Qt.NoPen)
        painter.setBrush(self._brush_background)
        painter.drawPath(path_content.simplified())


        # outline
        path_outline = QPainterPath()
        path_outline.addRoundedRect(-1, -1, self.width+2, self.height+2, self.edge_roundness, self.edge_roundness)
        painter.setBrush(Qt.NoBrush)
        if self.hovered:
            painter.setPen(self._pen_hovered)
            painter.drawPath(path_outline.simplified())
            painter.setPen(self._pen_default)
            painter.drawPath(path_outline.simplified())
        else:
            painter.setPen(self._pen_default if not self.isSelected() else self._pen_selected)
            painter.drawPath(path_outline.simplified())


from ThermodynamicCycles.FluidPort.FluidPort import FluidPort
from CoolProp.CoolProp import PropsSI
import pandas as pd 

class Object :
    def __init__(self):
        self.Inlet=FluidPort() 
        self.Outlet=FluidPort()
       
  
        self.Q_exp=0

         #Output Data
        self.df=[]

    def calculate(self):
        self.Outlet.fluid=self.Inlet.fluid
        #print(self.Inlet.h)
        self.Outlet.h = self.Inlet.h
        #print("Ho=",self.Outlet.h)
        self.Outlet.T=PropsSI('T','P',self.Outlet.P,'H',self.Outlet.h,self.Inlet.fluid)
     
        #print("self.Outlet.T °C=",self.Outlet.T-273.15)
        #print("valv self.Outlet.P=",self.Outlet.P)
        self.Outlet.S =PropsSI('S','P',self.Outlet.P,'H',self.Outlet.h,self.Inlet.fluid)
        
        
        self.Outlet.F=self.Inlet.F
        #self.Outlet.P=self.LP
        #pour calculate une detente avec travail exterieur :(=0)
        self.Q_exp=self.Inlet.F*(self.Outlet.h-self.Inlet.h)

        self.Outlet.calculate_properties()
        
        self.df = pd.DataFrame({'Expansion_Valve': [self.Inlet.fluid,self.Outlet.F,self.Outlet.T-273.15, self.Outlet.h/1000, self.Outlet.S/1000,], },
                      index = ['fluid','Outlet.F', "To(°C)", "Ho (kJ/kg)", "So(kJ/kg-K)", ])


       


from AHU.air_humide import air_humide
from AHU.air_humide import air_humide_NB
from AHU.AirPort.AirPort import AirPort


class Object:
    def __init__(self):
        self.Inlet=AirPort() 
        self.Outlet=AirPort()
        
        #parameter :
        self.P_drop=0
        
        '''Point de consignes'''
        self.w_target=8 
        self.T_target=0
        
        '''Température point de rosée de l air au contact de la Coil froide '''
        self.T_sat=7  
        '''Températures inlet et outlet'''
        self.T_inlet=0
        self.T_Outlet=0
        
        '''Déshumidification Type'''
        self.Déshutype="Droite_sat"
                
        """facteur de bypass"""
        self.FB=0.2
        self.w_sat=0
        self.h_sat=0
        # calcul Outlet Coil
       
        self.Q_th=12      
        self.Eff=0.8 #efficacité de la Coil froide
        
        self.F=0
        self.F_m3h=0
        self.F_dry=0 #Débit d'air sec
        
    def calculate(self):
        #print("self.Inlet.P=",self.Inlet.P,"air_humide.Air_Pv_sat(self.T_sat)=",air_humide.Air_Pv_sat(self.T_sat),"self.T_sat=",self.T_sat)
        self.w_sat=air_humide.Air_w(T_db=self.T_sat, RH=100, P=self.Inlet.P)
        self.h_sat=air_humide.Air_h(T_db=self.T_sat,w=self.w_sat)
        self.Outlet.w=self.w_target
        self.T_inlet=air_humide_NB.Air3_Tdb(self.Inlet.w/1000, self.Inlet.P, self.Inlet.h)
        self.F=self.Inlet.F
        #print("self.w_sat=",self.w_sat,"self.w_target=",self.w_target)
        
        '''Test que la consigne de déshu est située entre w de l'air humide et HA_sat à la surface de la Coil froide'''
        # if self.Déshutype="Droite_sat":
        if self.Inlet.w>=self.w_target and self.w_target>=self.w_sat:
          
            self.Eff=(self.Inlet.w-self.Outlet.w)/(self.Inlet.w-self.w_sat)
            self.FB=1-self.Eff
            self.Outlet.h=self.Inlet.h-self.Eff*(self.Inlet.h-self.h_sat)
            self.T_Outlet=air_humide_NB.Air3_Tdb(self.w_target/1000, self.Inlet.P, self.Outlet.h)
            # print('CAS 1')
            
            #'''cas où après la transormation, la température de l'air traité est supérieure à la température de consigne'''
           
            # if self.T_Outlet>=self.T_target:
            #     self.Inlet.h=self.Outlet.h
            #     self.T_inlet=self.T_Outlet
            #     self.Eff=(self.T_target-self.T_sat)/(self.T_inlet-self.T_sat)
            #     self.FB=1-self.Eff
            #     self.Outlet.h=self.h_sat+self.Eff*(self.Inlet.h-self.h_sat)
            #     self.Outlet.w=self.w_sat+self.Eff*(self.w_target-self.w_sat)
              
                # print('CAS 2')
                
                
             #Air exterieur compris entre HA_sat et HA_Target et T_air extérieur > T_target   
       
        elif self.Inlet.w>self.w_sat and self.Inlet.w<=self.w_target and self.T_inlet>=self.T_target:
            self.Eff=(self.T_target-self.T_sat)/(self.T_inlet-self.T_sat)
            self.FB=1-self.Eff
            self.Outlet.h=self.h_sat+self.Eff*(self.Inlet.h-self.h_sat)
            self.Outlet.w=self.w_sat+self.Eff*(self.w_target-self.w_sat)
            #self.Outlet.h=air_humide_NB.Air4_Hs(self.T_target, self.Inlet.P, self.Inlet.w/1000)
            #self.Outlet.w=self.Inlet.w
            # print('CAS 3')
            
        # '''Cas où l'w de l'air à traiter est inférieure à l'HA_sat à la surface de la Coil'''
        #         ''' Avec refroidissement sensible sans déshu '''
        elif self.Inlet.w<=self.w_sat and self.T_inlet>=self.T_target:
            self.Eff=(self.T_sat-self.T_target)/(self.T_inlet-self.T_sat)
            self.FB=1-self.Eff
            self.Outlet.h=air_humide_NB.Air4_Hs(self.T_target, self.Inlet.P, self.Inlet.w/1000)
            self.Outlet.w=self.Inlet.w
            #print('CAS 4')
            
            ''' sans aucune action de la Coil froide (Pas de traitement à faire) '''
        else:
            self.Outlet.h=self.Inlet.h
            self.Outlet.w=self.Inlet.w
            self.Eff=0
            self.FB=1-self.Eff
            # print('CAS 5') 
        
        
        
        
        self.Outlet.P=self.Inlet.P-self.P_drop
        #self.F_dry=(self.Inlet.F)/air_humide_NB.Air3_Vs(self.Inlet.w/1000,self.Inlet.P,self.Inlet.h) #[m3/s] / [m3/kg air sec]  = [kg air sec/s]
        self.F_dry=(self.Inlet.F)/(1+(self.Inlet.w/1000))
        #print("self.F_dry=",self.F_dry)
        self.Q_th=(self.Outlet.h-self.Inlet.h)*self.F_dry       
        #self.Outlet.F=self.F_dry*air_humide_NB.Air3_Vs(self.Outlet.w/1000,self.Outlet.P,self.Outlet.h) #[kg air sec/s] * [m3/kg air sec] =[m3/s]
        self.Outlet.F=self.F_dry*(1+(self.Outlet.w/1000))
        #print("self.Inlet.F=",self.Inlet.F)
        #print("self.Outlet.F=",self.Outlet.F)
        self.Outlet.F_dry=self.F_dry
        
        
 
           
            
            


